(game-module "friedrich"
  ;; This is not a standalone game, has only a test setup defined.
  (blurb "Type definitions for games set during the Seven Years War")
  (variants (world-size 45 30 1000))
)

(set see-all true) ; for debugging

(unit-type army (image-name "soldiers"))
(unit-type general (image-name "flag"))
(unit-type marshal (image-name "flag"))
(unit-type prince (image-name "flag"))
(unit-type king (image-name "crown"))
(unit-type depot (image-name "walltown"))
(unit-type town (image-name "town20"))
(unit-type fortress (image-name "fortress"))

(define a army)
(define g general)
(define m marshal)
(define p prince)
(define k king)
(define D depot)
(define * town)
(define @ fortress)

(material-type supplies (help "generic supplies"))

(terrain-type sea (color "sky blue") (char "."))
(terrain-type countryside (color "green") (char "+") (image-name "countryside"))
(terrain-type swamp (color "yellow green") (char "="))
(terrain-type forest (color "forest green") (char "%"))
(terrain-type hills (color "khaki") (char ">"))
(terrain-type mountains (color "sienna") (char "^"))
(terrain-type river (color "blue") (subtype border))

(define land (countryside swamp forest hills mountains))
(define leader (g m p k))
(define bases (D * @))

;;; Static relationships.

(table vanishes-on
  (u* sea true)
  )

; Armies carry leaders around and are propelled by them.

(add army capacity 1)
(add bases capacity 10)

(table unit-size-as-occupant
  (army bases 1)
  (leader army 1)
  (leader leader 1)
  (leader bases 0)
  (bases bases 100)  ; prevents putting bases inside each other
  )

;;; Leaders can carry each other around, higher ranks carrying lower.

(table unit-capacity-x
  (k (g m p) 4)
  (p (g m) 3)
  (m (g) 2)
  )

(table unit-size-in-terrain
  (u* t* 0)
  ;; (need to limit armies?)
  )

(add army storage 100)
(add bases storage 1000)

(table material-size-in-unit (u* supplies 1))

;;; Action parameters.

(add army acp-per-turn 1)  ; leader occupant should increase this as well as speed
(add leader acp-per-turn 6)

;;; Movement parameters.

(add army speed 1)
(add leader speed 6)

;(table mp-per-occupant
;  (army leader 5)
;  )

(add leader acp-per-move 100)

(table mp-to-enter-terrain
  (u* sea 99)
  (army (hills mountains) (2 99))
  ;; Leaders can take shortcuts through the mountains.
  (leader mountains 3)
  ;; Crossing rivers is hard for armies.
  (a river 1)
)

(add army free-mp 1)

;; 6 u* attack_moves

;; 5 s supply-line

;;; Construction parameters.

(add depot cp 10)

(table acp-to-create (army depot 1))

(table acp-to-build (army depot 1))

(table cp-per-build (army depot 1))

;;; Combat parameters.

(add army hp-max 40)

;(table acp-to-attack
;  )

(table hit-chance
  (army army 50)
  (army leader 50)
  )

(table damage
  (army army 1)
  (army leader 1)
  )

(table capture-chance
  (army army 25)
  (army leader 100)
  (army bases (100 10 1))
;  (depot (* @) (50 20)) ; should be a surrender-chance for this
  )

;;; Seasons.

(world (year-length 24))

(set calendar ("usual" "month" 1 2))

;;; Random generation; should be for testing only.

(add t* alt-percentile-min (  0  70  30  20  90  95  0))
(add t* alt-percentile-max ( 20  90  31  90  95 100  0))
(add t* wet-percentile-min (  0  50  50   0   0   0  0))
(add t* wet-percentile-max (100 100 100 100 100 100  0))

(set edge-terrain mountains)

(add u* start-with 1)  ; one of everything

(table favored-terrain
  (u* t* 0)
  (u* countryside 100)
  )

;;; (this is actually not necessary, since actual sides should be defined)

(side-library set
  ((name "Austria") (adjective "Austrian"))
  ((name "Prussia") (adjective "Prussian"))
  ((name "France") (adjective "French"))
  ((name "Russia") (adjective "Russian"))
  ((name "Swabia") (adjective "Swabian"))
  ((name "Hannover") (adjective "Hannoverian"))
  ((name "Saxony") (adjective "Saxon"))
  ((name "Sweden") (adjective "Swedish"))
  )

(independent-units (emblem-name "none"))  ; should be the default eventually

(game-module (notes (
  "This is a game inspired by the AH game Frederick the Great, which"
  "treats the Seven Years War in central Europe.  The number of unit types"
  "is small, and there are only a few special characteristics.  The game"
  "is basically one of maneuvering for position, since combat is pretty"
  "ineffective."
  ""
  "Map scale is 15 miles/cell, time is 2 weeks/turn."
  ""
  )))

(game-module (design-notes (
  "A number of items remain to be developed:"
  ""
  "Leaders speeding up armies."
  ""
  "Austrian zones of control."
  ""
  "Disable free moves and make combat cost high, to simulate the mutual"
  "consent to combat."
  ""
  "Do supply path stuff (4 hexes Coalition, 5 hexes Prussian)."
  ""
  "Have river effect on combat."

; how to do captured prisoners and prisoner exchange?
  )))
