(game-module "ww2-div-eur"
  (title "ww2-division-europe")
  (blurb "Defns for division-level WWII in Europe")
)

(unit-type a-inf (name "infantry(a)") (image-name "inf-div")
  (possible-sides "allies"))
(unit-type a-armor (name "armor(a)") (image-name "armor-div")
  (possible-sides "allies"))
(unit-type a-para (name "paratroops(a)") (image-name "para-div")
  (possible-sides "allies"))

(unit-type g-inf (name "infantry(g)") (image-name "inf-div")
  (possible-sides "axis"))
(unit-type g-armor (name "armor(g)") (image-name "armor-div")
  (possible-sides "axis"))
(unit-type g-para (name "paratroops(g)") (image-name "para-div")
  (possible-sides "axis"))
(unit-type pzgren (name "panzergrenadiers") (image-name "pzgren-div")
  (possible-sides "axis"))
(unit-type metro (image-name "metro-div")
  (possible-sides "axis"))
(unit-type cd (image-name "cd-div")
  (possible-sides "axis"))

(unit-type hq (image-name "hq-div")
  )

(define ground-types (a-inf a-armor a-para g-inf g-armor g-para pzgren metro cd hq))

(define non-mot-types (a-inf g-inf))

(define mot-types (a-armor g-armor))

(unit-type transport (image-name "ap"))
(unit-type battleship (image-name "bb"))

(define ship-types (transport battleship))

(unit-type airbase)
(unit-type port)
(unit-type coast-defense)
(unit-type town (image-name "town20"))
(unit-type factory)

(define places (airbase port coast-defense town factory))

;;; Abstracted aircraft. (should have diff types?)

(material-type org
  (help "organization - important to do things, acquired from HQs"))
(material-type aircraft
  (help "attached aircraft used for support"))

; sea clr brkn rough mtn low urb road rail river rhine

(terrain-type sea
  (extensions (water true)))
(terrain-type swamp)
(terrain-type clear)
(terrain-type bocage)
(terrain-type hills)
(terrain-type mountains)
(terrain-type urban)
(terrain-type road (subtype connection))
(terrain-type railroad (subtype connection))
(terrain-type river (subtype border))
(terrain-type Rhine-river (subtype border))

(define land (swamp clear bocage hills mountains urban))

(define cell-types (sea swamp clear bocage hills mountains urban))

(add sea elevation-min 0)
(add sea elevation-max 0)

;;; Static relationships.

(table vanishes-on
  (ground-types sea true)
  (ship-types land true)
  )

(add t* capacity 3)

(add transport capacity 6)

;;; Action parameters.

(add u* acp-per-turn 1)
(add ground-types acp-per-turn 7)
(add places acp-per-turn 0)

;;; Movement.

(add u* speed 1)
(add ground-types speed 7)
(add ship-types speed 70)
(add places speed 0)

(table mp-to-enter-terrain
  (mot-types hills 2)
  (ground-types mountains 2)
  (mot-types mountains 4)
  (hq mountains 1)
  (ground-types sea 999)
  (ship-types land 999)
  )

;;; Combat.

(add ground-types hp-max 10)

(table hit-chance
  (u* u* 50)
  (hq u* 0)
  )

(table damage
  (u* u* 1)
  (hq u* 0)
  )

;; Battleships can reach inland a little ways.

(add battleship range 2)

;; (could have units capable of calling down airstrikes in their vicinity)

;;; Definitions used for testing.

;;; Some defns for the fractal percentile generator.

(set alt-blob-density 10000)
(set alt-blob-size 20)
(set alt-smoothing 6)
(set wet-blob-density 2000)
(set wet-blob-size 100)

; sea low clr brkn rough mtn urb road rail river rhine

(add t* alt-percentile-min (  0   30  30   30   70  90   30  0 0 0 0))
(add t* alt-percentile-max ( 30   35 70   70   90  100  70  0 0 0 0))
(add t* wet-percentile-min (  0   50   0   50   0    0    45  0 0 0 0))
(add t* wet-percentile-max (100 100  45 100 100 100  50 0 0 0 0))

;;; River generation.

(add (clear hills mountains) river-chance (1000 1000 2000))

(set river-sink-terrain sea)

;;; Road generation.

;(table road-into-chance
;  (t* plains 100)
;  (plains (desert forest mountains) (50 40 20))
 ; )

(set country-radius-min 6)

(add (sea clear) country-terrain-min (5 10))

(add u* start-with 1)  ; one of everything

(table favored-terrain
  (u* (sea swamp) 0) 
  (ship-types sea 100)
  (ship-types land 0)
  )

(side-library set
  ((name "Allies") (adjective "Allied") (class "allied"))
  ((name "Axis") (adjective "Axis") (class "axis"))
)

(world 2500 (year-length 52))

(game-module (design-notes (
  "The map scale is 10 miles/hex, game time is 1 week/turn."
  ""
  "This game design covers division-level operations in Europe during WWII."
  "Players take on the roles of army commanders but not political leaders;"
  "the game does not include political factors."
  )))

#|

AtR uses a 50x50 map for N. France + Rhine Valley.

Organization is consumed by movement/combat, supplied by HQs

Need general concept of comm line?

Coast defense must be destroyed or captured to establish beachhead.
Map will have row all along water?  Different quality or size units
at different points?

HQs should auto-retreat.  Inability to retreat results in capture.

Parachutists have restrictions on initial placement (why?), won't
drop further than 10 hexes from friendlies (even on invasion?)
(use control radius to impl)

Mulberry is like a stationary transport, placed in the water by ships,
has better unit/supply transfer parms.

Can build fortresses (very slowly), units can be inside or outside.

Have organization material increase effectiveness of combat (and movement).

Admin movement should use trains/trucks (introduce as units with
huge move allowance?).

All units have high prob to surrender - use ZOC to calculate.

|#
