/* Copyright (c) 1992, 1993  Stanley T. Shebs. */
/* This program may be used, copied, modified, and redistributed freely */
/* for noncommercial purposes, so long as this notice remains intact. */

/* Main Mac interface. */

#include "conq.h"
#include "mac.h"

/* The console is a useful debugging hack.  When not in use, anything that uses stdout
   should be disabled. */

#ifdef USE_CONSOLE
#ifdef THINK_C
#include <console.h>
#include <Folders.h>
#else
printf() {}
#endif
#endif

/* (from DTS sample code) */
/* kOSEvent is the event number of the suspend/resume and mouse-moved events sent
   by MultiFinder. Once we determine that an event is an osEvent, we look at the
   high byte of the message sent to determine which kind it is. To differentiate
   suspend and resume events we check the resumeMask bit. */

#define	kOSEvent				app4Evt	/* event used by MultiFinder */
#define	kSuspendResumeMessage	1		/* high byte of suspend/resume event message */
#define	kResumeMask				1		/* bit of message field for resume vs. suspend */
#define	kMouseMovedMessage		0xFA	/* high byte of mouse-moved event message */

extern CursHandle allmovecursor;
extern CursHandle watchcursor;

int modaltool = 0;

int inbackground;

/* This is the list of maps that we're using. */

struct a_map *maplist;

/* This is the list of lists. */

struct a_list *listlist;

/* This is the list of unit closeups. */

struct a_unit_closeup *unitcloseuplist;

struct a_side_closeup *sidecloseuplist;	 /* chain of side closeups */

/* This flag indicates whether the image etc resource file(s) were found. */

int foundresourcefile = FALSE;

/* The usual width of a scrollbar. */

int sbarwid = 15;

int defaultdrawgrid = TRUE;		   /* Display grid on the map? */
int defaultdrawnames = FALSE;		   /* Display unit names/numbers on the map? */

/* True if we're going to use WaitNextEvent. */

int useWNE = FALSE;

/* Rectangle that constrains window dragging. */

Rect dragrect;

/* Rectangle that constrains window resizing. */

Rect sizerect;

/* This is the side that is using this Mac as its display. */

Side *dside = NULL;

/* This points to a spare block of memory that is freed so shutdown code can
   use it (no guarantee that it will tho). */

char *spare;

/* This is true if savable prefs etc have been saved since being changed. */
/* (always true for now, nothing being remembered) */

int interfacestatesafe = TRUE;

int didamove = FALSE;

int moveonclick = FALSE;

int wasingame = TRUE;

int suppresswarnings = FALSE;

/* Set to true if Color QuickDraw is installed. */

int hasColorQD;

int maxscreendepth = 1;

/* The main Mac program. */

main()
{
	/* Do the most basic Macintosh setup. */
	init_toolbox();
	init_cursors();
	init_menus();
	init_rects();
#ifdef USE_CONSOLE
	/* Make sure the console window is up before anybody tries to write to it. */
	freopenc(NULL, stdout);
	freopenc(stdout, stderr);
#endif
	/* Acquire Mac-specific files (preferences and resources). */
	get_files();
	/* Put the Xconq kernel into a known state. */
	clear_game_modules();
	init_data_structures();
	init_library_path();
	/* A hack to ensure some memory available for error handling. */
	spare = (char *) xmalloc(1000);
	/* Show the splash screen, and go to an appropriate dialog for each of the
	   possible responses. */
	switch (do_about_box()) {
		case diAboutNew:
			new_game_dialog();
			break;
		case diAboutOpen:
			open_game_dialog();
			break;
		case diAboutConnect:
			connect_game_dialog();
			break;
		case diAboutQuit:
			/* Quit directly, no confirmation needed. */
			ExitToShell();
			break;
	}
	/* All essential init done, jump into the main event loop. */
	event_loop();
}

/* (just a place holder for now) */

connect_game_dialog()
{
	LocationNameRec foo;
	PortInfoRec bar;
	
	PPCBrowser("\pEventually you'll be able to choose an Xconq game from here!",
			   "\pXconq", 0, &foo, &bar, NULL, "\p?");
}

/* Do the usual Mac setup calls. */

init_toolbox()
{
	SysEnvRec se;

	InitGraf(&QD(thePort));
	InitFonts();
	FlushEvents(everyEvent, 0);
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	InitCursor();

	SysEnvirons(2, &se);
	hasColorQD = se.hasColorQD;
	DGprintf("%s Color QuickDraw\n", (hasColorQD ? "Using" : "Not using"));
	/* (should compute the max possible depth of physical devices) */
	maxscreendepth = 1;
}

/* Set up the generic dragging and sizing rects. */

init_rects()
{
	RgnHandle screenrgn;

	screenrgn = GetGrayRgn();
	dragrect = (*screenrgn)->rgnBBox;
	SetRect(&sizerect, 40, 40, (*screenrgn)->rgnBBox.right,  (*screenrgn)->rgnBBox.bottom);
}

/* Open and/or load any files that we might need, such as preferences
   and resources. */

get_files()
{
	short vref, refnum;
	long dirid;
	Str255 filename;
	FSSpec spec;

	/* Load up any preferences. */
	GetIndString(filename, sFilenames, siPreferences);
#ifndef THINK_C
	if (FindFolder(kOnSystemDisk, kPreferencesFolderType, kDontCreateFolder,
				   &vref, &dirid) == noErr) {
		if (FSMakeFSSpec(vref, dirid, filename, &spec) == noErr) {
			refnum = FSpOpenResFile(&spec, fsCurPerm);
			CloseResFile(refnum);
		}
	}
#endif
	/* Look for and open game library resource file(s). */
	foundresourcefile = FALSE;
	GetIndString(filename, sFilenames, siResources);
	if (OpenResFile(filename) != -1) {
		foundresourcefile = TRUE;
	}
	/* Note that we don't complain yet if the resource files are missing,
	   since we don't yet know whether we actually need anything from them.
	   (Images etc might be built into app or game module, for instance.) */
}

/* Since Mac programs effectively take over the entire machine, we depend on
   this event loop to handle everything that might come along.  */

event_loop()
{
	WindowPtr win;
	Rect r;
	RgnHandle cursorRgn;
	Boolean gotevent;
	EventRecord	event;
	Point mouse;
	DialogPtr dialog;
	short itemhit;

	/* Figure out if the WaitNextEvent Trap is available. */
	useWNE = (NGetTrapAddress(0x60, ToolTrap) != NGetTrapAddress(0x9f, ToolTrap));
	/* Pass WNE an empty region the 1st time thru. */
	cursorRgn = NewRgn();
	/* Loop forever. */
	while (1) {
		/* Use WaitNextEvent if it is available, otherwise GetNextEvent. */
		if (useWNE) {
			get_global_mouse(&mouse);
			adjust_cursor(mouse, cursorRgn);
			gotevent = WaitNextEvent(everyEvent, &event, 0L, cursorRgn);
		} else {
			SystemTask();
			gotevent = GetNextEvent(everyEvent, &event);
		}
		/* First decide if the event is for a dialog or is just any old event. */
		if (FrontWindow() != nil && IsDialogEvent(&event)) {
			/* Handle all the modeless dialogs here. */
			if (DialogSelect(&event, &dialog, &itemhit)) {
				if (dialog == helpwin) {
					hit_help_dialog(dialog, itemhit, &event);
				} else if (dside != NULL) {
					if (hit_in_side_closeup(dialog, itemhit, &event)) {
					} else {
						DGprintf("random dialog hit?\n");
					}
				} else {
					DGprintf("really random dialog hit?\n");
				}
			} else {
				/* ??? */
			}
		} else if (gotevent) {
			/* Make sure we have the right cursor before handling the event. */
			adjust_cursor(event.where, cursorRgn);
			didamove = FALSE;
			/* Now interpret the event. */
			do_event(&event);
			/* Give the kernel a chance to run things. */
			/* Don't automatically go to a watch cursor, since run_game often
			   returns very quickly. */
			run_game(4);
			/* If kernel execution has used up some unit moves, and the user
			   has asked for the auto-select option, then we might have to change
			   the selected unit here. */
			maybe_select_next_unit();
		}
	}
}

get_global_mouse(mouse)
Point *mouse;
{
	EventRecord	evt;
	
	OSEventAvail(0, &evt);
	*mouse = evt.where;
}

Point lastmouse;

char mouseoverbuf[100];

/* Change the cursor to reflect the context. */

adjust_cursor(mouse, region)
Point mouse;
RgnHandle region;
{
	Map *map;
	CursPtr curs = &QD(arrow);
	GrafPtr oldport;

	if ((map = (Map *) map_from_window(FrontWindow())) != NULL) {
		if (mouse_in_map(mouse, map)) {
			if (modaltool) {
				curs = *opencrosscursor;
#ifdef DESIGNERS
			} else if (dside->designer && tooltype != notool) {
				curs = (CursPtr) adjust_designer_cursor(mouse, region);
#endif DESIGNERS
			} else if (moveonclick) {
				int x, y, approxdir = 1;
				Unit *unit = NULL;

				if (map->numselections == 1 && (unit = map->selections[0]) != NULL) {
					/* Calculate the approx dir to here from selected unit. */
					GetPort(&oldport);
					SetPort(map->window);
					GlobalToLocal(&mouse);
					SetPort(oldport);
					nearest_cell(map, mouse.h, mouse.v, &x, &y);
					approxdir = approx_dir(x - unit->x, y - unit->y);
					/* (should use 12 dirs, for smoothness?) */
					curs = *(movecursors[approxdir]);
				}
				if (map->numselections > 1) {
					curs = *allmovecursor;
				}
			}
		}
		/* This isn't really cursor adjustment, but this is the right place. */
		if (map->toph > 0 && !EqualPt(mouse, lastmouse)) {
			int x, y, inmap;
			Unit *unit = NULL;
			GrafPtr oldport;
			extern char *mouseover;

			GetPort(&oldport);
			SetPort(map->window);
			GlobalToLocal(&mouse);
			inmap = PtInRect(mouse, &(map->contentrect));
			if (!inmap) {
				sprintf(tmpbuf, "");
			} else {
				oneliner(map, mouse.h, mouse.v);
			}
			if (strcmp(tmpbuf, mouseoverbuf) != 0) {
				strcpy(mouseoverbuf, tmpbuf);
				mouseover = mouseoverbuf;
				draw_top_line(map);
			}
			SetPort(oldport);
			lastmouse = mouse;
		}
	}
	SetCursor(curs);
}

mouse_in_map(mouse, map)
Point mouse;
Map *map;
{
	Rect tmprect;
	GrafPtr oldport;
	extern int conwid;

	GetPort(&oldport);
	SetPort(map->window);
	tmprect = map->window->portRect;
	tmprect.left += conwid;
	tmprect.bottom -= sbarwid;  tmprect.right -= sbarwid;
	LocalToGlobal(&top_left(tmprect));
	LocalToGlobal(&bottom_right(tmprect));
	SetPort(oldport);
	return PtInRect(mouse, &tmprect);
}

/* Decipher an event. */

do_event(event)
EventRecord *event;
{
	short part, err;
	WindowPtr win;
	Boolean hit;
	char key;
	Point pnt;
	GrafPtr oldport;

	switch (event->what) {
		case mouseDown:
			/* See if the click happened in a special part of the screen. */
			part = FindWindow(event->where, &win);
			switch (part) {
				case inMenuBar:
					adjust_menus();
					do_menu_command(MenuSelect(event->where));
					break;
				case inSysWindow:
					SystemClick(event, win);
					break;
				case inContent:
					if (win != FrontWindow()) {
						/* Bring the clicked-on window to the front. */
						SelectWindow(win);
						/* Fix the menu to match the new front window. */
						adjust_menus();
						/* We always want to discard the event now, since clicks in a
						   windows are often irreversible actions. */
					} else {
						/* Mouse clicks in the front window do something useful. */
						do_mouse_down(win, event);
					}
					break;
				case inDrag:
					/* Standard drag behavior, no tricks necessary. */
					DragWindow(win, event->where, &dragrect);
					break;
				case inGrow:
					grow_window(win, event->where);
					break;
				case inZoomIn:
				case inZoomOut:
					zoom_window(win, event->where, part);
					break;
				case inGoAway:
					close_window(win);
					break;
			}
#ifdef DEBUGGING
			/* This forces output into the file. */
			if (Debug || DebugG || DebugM) do_debugging(TRUE);
#endif
			break;
		case keyDown:
		case autoKey:
			key = event->message & charCodeMask;
			/* Check for menukey equivalents. */
			if (event->modifiers & cmdKey) {
				if (event->what == keyDown) {
					adjust_menus();
					do_menu_command(MenuKey(key));
				}
			} else {
				if (event->what == keyDown) {
					/* Random keypress, interpret it. */
					do_keyboard_command(key);
				}
			}
			break;
		case activateEvt:
			activate_window((WindowPtr) event->message, event->modifiers & activeFlag);
			break;
		case updateEvt:
			update_window((WindowPtr) event->message);
			break;
		case diskEvt:
			/*	Call DIBadMount in response to a diskEvt, so that the user can format
				 a floppy. (from DTS Sample) */
			if (HiWord(event->message) != noErr) {
				SetPt(&pnt, 50, 50);
				err = DIBadMount(pnt, event->message);
			}
			break;
		case kOSEvent:
		    /* Grab only a single byte. */
			switch ((event->message >> 24) & 0x0FF) {
				case kMouseMovedMessage:
					break;
				case kSuspendResumeMessage:
					inbackground = !(event->message & kResumeMask);
					activate_window(FrontWindow(), !inbackground);
					break;
			}
			break;
#ifdef THINK_C
		case kHighLevelEvent:
			/* (should add all the AE stuff here) */
			break;
#endif
		default:
			break;
	}
}

grow_window(win, where)
WindowPtr win;
Point where;
{
	long winsize;
	GrafPtr oldport;

	winsize = GrowWindow(win, where, &sizerect);
	if (winsize != 0) {
		GetPort(&oldport);
		SetPort(win);
		if (map_from_window(win)) {
			grow_map(map_from_window(win), LoWord(winsize), HiWord(winsize));
		} else if (list_from_window(win)) {
			grow_list(list_from_window(win), LoWord(winsize), HiWord(winsize));
		} else if (buildwin == win) {
			grow_construction(winsize);
		} else {
			/* Generic window sizing. */
			EraseRect(&win->portRect);
			SizeWindow(win, LoWord(winsize), HiWord(winsize), 1);
			InvalRect(&win->portRect);
		}
		SetPort(oldport);
	}
}

zoom_window(win, where, part)
WindowPtr win;
Point where;
short part;
{
	GrafPtr oldport;

	if (TrackBox(win, where, part)) {
		GetPort(&oldport);
		/* The window must be the current port. (ZoomWindow bug) */
		SetPort(win);
		if (map_from_window(win)) {
			zoom_window_map(map_from_window(win), part);
		} else if (list_from_window(win)) {
			zoom_window_list(list_from_window(win), part);
		} else {
			/* Generic window zooming. */
			EraseRect(&win->portRect);
			ZoomWindow(win, part, true);
			InvalRect(&win->portRect);
		}
		SetPort(oldport);
	}
}

close_window(win)
WindowPtr win;
{
	if (is_da_window(win)) {
		CloseDeskAcc(((WindowPeek) win)->windowKind);
	} else if (is_app_window(win)) {
		/* Remove from the windows menu (OK to call even if window not in menu). */
		remove_window_menu_item(win);
		if (win == buildwin) {
			HideWindow(win);
#ifdef DESIGNERS
		} else if (win == designwin) {
			/* Closing the design palette implies we're done designing. */
			disable_designing();
#endif /* DESIGNERS */
		} else {
			HideWindow(win);
/*			CloseWindow(win);
			DisposPtr((Ptr) win);  */
		}
	}
}

/* This just dispatches to the appropriate window handler. */

do_mouse_down(window, event)
WindowPtr window;
EventRecord *event;
{
	Point mouse;
	Map *map;
	List *list;

	if (is_app_window(window)) {
		SetPort(window);
		mouse = event->where;
		GlobalToLocal(&mouse);
		/* Locate the interface object that this is on. */
		if ((map = map_from_window(window)) != NULL) {
			do_mouse_down_map(map, mouse, event->modifiers); 
		} else if ((list = list_from_window(window)) != NULL) {
			do_mouse_down_list(list, mouse, event->modifiers); 
		} else if (buildwin == window) {
			do_mouse_down_construction(mouse, event->modifiers);
#ifdef DESIGNERS
		} else if (designwin == window) {
			do_mouse_down_design(mouse, event->modifiers);
#endif /* DESIGNERS */
		} else {
		}
	}
}

/* Bringing a window to the front may entail messing with the menu. */

activate_window(win, activate)
WindowPtr win;
int activate;
{
	Map *map;

	if ((map = map_from_window(win)) != NULL) {
		activate_map(map, activate);
	} else if (win == buildwin) {
		LActivate(activate, buildunitlist);
		LActivate(activate, buildtypelist);
	} else {
		DGprintf("%sactivating random window\n", (activate ? "" : "de"));
	}
	if (activate) {
		/* It's convenient to make the activated window also be the current GrafPort. */
		SetPort(win);
	}
	adjust_menus();
}

/* Update a given window.  This is the main routine that causes drawing into
   all the different kinds of windows. */

update_window(win)
WindowPtr win;
{
	int startticks, drawticks = 0;
	GrafPtr oldport;
	Map *map;
	List *list;
	SideCloseup *sidecloseup;
	UnitCloseup *unitcloseup;

	GetPort(&oldport);
	SetPort(win);
	BeginUpdate(win);
	if ((map = map_from_window(win)) != NULL) {
		erase_map(map);
		DrawControls(win);
		DrawGrowIcon(win);
		startticks = TickCount();
		draw_map(map);
		drawticks = TickCount() - startticks;
	} else if ((list = list_from_window(win)) != NULL) {
		draw_list(list);
		DrawControls(win);
		DrawGrowIcon(win);
	} else if ((unitcloseup = unit_closeup_from_window(win)) != NULL) {
		draw_unit_closeup(unitcloseup);
		DrawControls(win);
	} else if ((sidecloseup = side_closeup_from_window(win)) != NULL) {
		draw_side_closeup(sidecloseup);
		DrawControls(win);
	} else if (gamewin == win) {
		draw_game();
	} else if (buildwin == win) {
		draw_construction();
		DrawControls(win);
		DrawGrowIcon(win);
	} else if (historywin == win) {
		draw_history();
		DrawControls(win);
		DrawGrowIcon(win);
#ifdef DESIGNERS
	} else if (designwin == win) {
		draw_design_window();
#endif /* DESIGNERS */
	} else {
#ifdef USE_CONSOLE
		/* Warn about unhandled windows, but don't use an alert... */
		printf("Update of window %x?\n", win);
#endif
	}
	EndUpdate(win);
	SetPort(oldport);
#ifdef USE_CONSOLE
	printf("draw_map took %f seconds\n", drawticks / 60.0);
#endif
}

maybe_select_next_unit()
{
	extern int autoselect;
	Map *map;

	if (ingame && autoselect && didamove) {
		if ((map = map_from_window(FrontWindow())) != NULL) {
			select_next_mover(map);
		}
	}
}

/* A warning just gets displayed, no other action is taken. */

init_warning(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
	char buf[BUFSIZE], buf2[BUFSIZE];

	if (suppresswarnings) return;
#ifdef USE_CONSOLE
	fprintf(stderr, "Warning: ");
	fprintf(stderr, str, a1, a2, a3);
	fprintf(stderr, "\n");
	fflush(stderr);
#endif
	/* Cursor may be weird from loading, reset it. */
	SetCursor(&qd.arrow);
	sprintf(buf, str, a1, a2, a3);
	c2p(buf, buf2);
	ParamText(buf2, "\p", "\p", "\p");
	switch (CautionAlert(aInitWarning, nil)) {
		case 1:
			/* Just keep going, hope that warning was a false alarm. */
			if (0 /* option key or some such */) {
				suppresswarnings = TRUE;
			}
			break;
		case 2:
			/* (should undo everything and blast back to initial choices) */
			ExitToShell();
			break;
	}
}

/* An init error is not necessarily fatal, but we still have to start over. */

init_error(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
	char buf[BUFSIZE], buf2[BUFSIZE];

	/* Make some space available just in case... */
	free(spare);
#ifdef USE_CONSOLE
	fprintf(stderr, "Error: ");
	fprintf(stderr, str, a1, a2, a3);
	fprintf(stderr, "\n");
	fflush(stderr);
#endif
	/* Cursor may be weird from loading, reset it. */
	SetCursor(&qd.arrow);
	sprintf(buf, str, a1, a2, a3);
	c2p(buf, buf2);
	ParamText(buf2, "\p", "\p", "\p");
	StopAlert(aInitError, nil);
	/* This is a bad time to die, no way to recover.  Fortunately,
	   it's not a big loss, since there's no game yet to lose,
	   and so we can just exit directly. */
	ExitToShell();
}

/* Runtime warnings are for when it's important to bug the players,
   but doesn't necessarily mean imminent danger of a crash. */

run_warning(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
	char buf[BUFSIZE], buf2[BUFSIZE];

	if (suppresswarnings) return;
#ifdef USE_CONSOLE
	fprintf(stderr, "\nWarning: ");
	fprintf(stderr, str, a1, a2, a3);
	fprintf(stderr, "\n");
	fflush(stderr);
#endif
	sprintf(buf, str, a1, a2, a3);
	c2p(buf, buf2);
	ParamText(buf2, "\p", "\p", "\p");
	switch (CautionAlert(aRunWarning, nil)) {
		case 1:
			if (0 /* option key?? */) {
				suppresswarnings = TRUE;
			}
			break;
		case 2:
			save_the_game(TRUE);
			ExitToShell();
			break;
		case 3:
			/* Just blast out of here. */
			ExitToShell();
			break;
	}
}

/* An run error is fatal, but allow an emergency save, might be able to salvage. */

run_error(str, a1, a2, a3)
char *str;
long a1, a2, a3;
{
	char buf[BUFSIZE], buf2[BUFSIZE];

	/* Make some space available just in case... */
	free(spare);
#ifdef USE_CONSOLE
	fprintf(stderr, "\nError: ");
	fprintf(stderr, str, a1, a2, a3);
	fprintf(stderr, "\n");
	fflush(stderr);
#endif
	sprintf(buf, str, a1, a2, a3);
	c2p(buf, buf2);
	ParamText(buf2, "\p", "\p", "\p");
	switch (StopAlert(aRunError, nil)) {
		case 1:
			break;
		case 2:
			save_the_game(TRUE);
			break;
	}
	/* We're outta here - just ahead of scrambled heaps and dangling ptrs! */
	ExitToShell();
}

/* This is true when a side has a display that may be safely written to. */

active_display(side)
Side *side;
{
	return (side && side->ui && side->ui->active);
}

/* The Mac never has any display buffers to flush (?). */

flush_display_buffers() {}

record_real_start_time() {}

/* Detect types of windows. */

is_da_window(win)
WindowPtr win;
{
	return (win != nil && ((WindowPeek) win)->windowKind < 0);
}

is_app_window(win)
WindowPtr win;
{
	return (win != nil && ((WindowPeek) win)->windowKind >= 0);
}

char *
newsfile_name()
{
	return "Xconq News"; /* get from resource */
}

char *
savefile_name()
{
	return "Saved Game"; /* get from resource */
}

FILE *
open_library_file(module)
Module *module;
{
	FILE *fp = NULL;
	
	/* Can't open anonymous library modules. */
	if (module->name == NULL) return NULL;
	/* Generate library pathname. */
	make_pathname(xconqlib, module->name, "g", spbuf);
	/* Now try to open the file. */
	if ((fp = fopen(spbuf, "r")) != NULL) {
		/* Remember the filename where we found it. */
		module->filename = copy_string(spbuf);
	}
	return fp;
}

FILE *
open_explicit_file(module)
Module *module;
{
	char fullname[255];
	FILE *fp;

	/* (should use value of hook to get the vrefnum) */
	sprintf(fullname, ":%s", module->filename);
	if ((fp = fopen(fullname, "r")) != NULL) {
		return fp;
	}
	sprintf(fullname, "%s%s", ":lib:", module->filename);
	if ((fp = fopen(fullname, "r")) != NULL) {
		return fp;
	}
    return NULL;
}

make_pathname(path, name, extn, pathbuf)
char *path, *name, *extn, *pathbuf;
{
    sprintf(pathbuf, "");
    if (!empty_string(path)) {
	sprintf(pathbuf+strlen(pathbuf), "%s:", path);
    }
    sprintf(pathbuf+strlen(pathbuf), "%s", name);
    /* Don't add a second identical extension, but do add if extension
       is different (in case we want "foo.12" -> "foo.12.g" for instance) */
    if (strrchr(name, '.') && strcmp(strrchr(name, '.')+1, extn) == 0)
      return;
    if (!empty_string(extn)) {
	sprintf(pathbuf+strlen(pathbuf), ".%s", extn);
    }
}

int nothing() {}

/* Kernel callback to update info about the given side. */

update_side_display(side, side2, rightnow)
Side *side, *side2;
int rightnow;
{
	GrafPtr oldport;
	extern int mayseeall;

	if (active_display(side) && side2 != NULL) {
		GetPort(&oldport);
		if (gamewin != nil /* && win is visible */) {
			SetPort(gamewin);
			draw_side_status(side2);
		}
		/* (should do side closeups etc) */
		SetPort(oldport);
		if (side2 == dside && !side->ingame && wasingame) {
			/* (should be able to quit from here) */
			CautionAlert(aOutOfGame, nil); 
			wasingame = FALSE;
			mayseeall = TRUE;
		}
	}
}

/* Kernel callback to show the current turn. */

update_turn_display(side, rightnow)
Side *side;
int rightnow;
{
	GrafPtr oldport;
	extern int autoselect;
	Map *map;

	if (active_display(side)) {
		GetPort(&oldport);
		if (gamewin != nil /* && win is visible */) {
			SetPort(gamewin);
			draw_game_date();
		}
		for_all_maps(map) {
			if (map->toph > 0) {
				SetPort(map->window);
				draw_top_line(map);
			}
		}
		SetPort(oldport);
	}
}

/* Callback that gets run once after all turn setup is done but before any movement. */

update_action_display(side, rightnow)
Side *side;
int rightnow;
{
	GrafPtr oldport;
	extern int autoselect;
	Map *map;

	if (active_display(side)) {
		GetPort(&oldport);
		for_all_maps(map) {
			draw_selections(map);
			if (autoselect && map->window == FrontWindow()) {
				unselect_all(map);
				select_next_actor(map);
			}
		}
		SetPort(oldport);
	}
}

update_action_result_display(side, unit, rslt, rightnow)
Side *side;
Unit *unit;
int rslt, rightnow;
{
    if (active_display(side)) {
    	/*DG*/printf("%s %s result is %s\n",
    			unit_desig(unit),
    			action_desig((unit->act ? &(unit->act->nextaction) : NULL)),
    			hevtdefns[rslt].name);
    	/* (should handle errors specially) */
    }
}

update_event_display(side, hevt)
Side *side;
HistEvent *hevt;
{
	extern int mayseeall;

	if (active_display(side)) {
		switch (hevt->type) {
		  case H_SIDE_LOST:
		    if (((Side *) hevt->data) == side) {
		    	lost_game_dialog();
		    	mayseeall = TRUE;
		    } else {
		    	/* indicate that other side lost */
		    }
		  	break;
		  case H_SIDE_WON:
		    if (((Side *) hevt->data) == side) {
		    	won_game_dialog();
		    	mayseeall = TRUE;
		    } else {
		    	/* indicate that other side won */
		    }
		  	break;
		  default:
		  	/* No special display. */
		  	break;
		}
	}
}

update_fire_at_display(side, unit, unit2, m, rightnow)
Side *side;
Unit *unit, *unit2;
int m, rightnow;
{
	int i, sx1, sy1, sw1, sh1, sx2, sy2, sw2, sh2;
	Map *map;
	GrafPtr oldport;

	if (active_display(side)) {
		for (i = 0; i < 8; ++i) {
			for_all_maps(map) {
				GetPort(&oldport);
				SetPort(map->window);
				PenMode(patXor);
				if (map->hw >= 8) PenSize(2, 2);
				xform_unit(map, unit, &sx1, &sy1, &sw1, &sh1);
				xform_unit(map, unit2, &sx2, &sy2, &sw2, &sh2);
				MoveTo(sx1+sw1/2, sy1+sh1/2);  LineTo(sx2+sw2/2, sy2+sh2/2);
				PenNormal();
				SetPort(oldport);
			}
		}
	}
}

/* Update any displayed info about the given unit. */

update_unit_display(side, unit, rightnow)
Side *side;
Unit *unit;
int rightnow;
{
	UnitCloseup *unitcloseup;

	if (active_display(side) && unit != NULL) {
		if (1 /* unit visible to side in any way */) {
			update_cell_display(side, unit->x, unit->y, TRUE);
		}
		update_unit_in_lists(unit);
		if ((unitcloseup = find_unit_closeup(unit)) != NULL) {
			draw_unit_closeup(unitcloseup);
		}
		if (unit->side != NULL && unit->act != NULL) {
			update_side_display(side, unit->side, rightnow);
		}
		if (buildwin != nil) {
			update_construction_unit_list(unit);
		}
	}
}

update_clock_display(side)
Side *side;
{
	if (active_display(side)) {
		/* (should set up clock displays) */
	}
}

won_game_dialog()
{
	int done = FALSE;
	Str255 tmpstr;
	short ditem;
	WindowPtr win;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	win = GetNewDialog(dWinGame, NULL, (DialogPtr) -1L);
	ShowWindow(win);
	while (!done) {
		draw_default_button(win, diWinGameQuit);
		ModalDialog(NULL, &ditem);
		switch (ditem) {
			case diWinGameQuit:
				ExitToShell();
				break;
			case diWinGameKeepGoing:
				done = TRUE;
				break;
			default:
				break;
		}
	}
	DisposDialog(win);
}

lost_game_dialog()
{
	int done = FALSE;
	Str255 tmpstr;
	short ditem;
	WindowPtr win;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	win = GetNewDialog(dLoseGame, NULL, (DialogPtr) -1L);
	ShowWindow(win);
	while (!done) {
		draw_default_button(win, diLoseGameQuit);
		ModalDialog(NULL, &ditem);
		switch (ditem) {
			case diLoseGameQuit:
				ExitToShell();
				break;
			case diLoseGameKeepGoing:
				done = TRUE;
				break;
			default:
				break;
		}
	}
	DisposDialog(win);
}

/* General routine to outline the given item of a given dialog. */

draw_default_button(dialog, ditem)
DialogPtr dialog;
short ditem;
{
	GrafPtr oldport;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	GetPort(&oldport);
	SetPort(dialog);
	GetDItem(dialog, ditem, &itemtype, &itemhandle, &itemrect);
	PenSize(3, 3);
	InsetRect(&itemrect, -4, -4);
	FrameRoundRect(&itemrect, 16, 16);
	PenNormal();
	SetPort(oldport);
}


force_update(win)
WindowPtr win;
{
	GrafPtr oldport;

	if (win == nil) return;
	GetPort(&oldport);
	SetPort(win);
	EraseRect(&win->portRect);
	InvalRect(&win->portRect);
	SetPort(oldport);
}

force_overall_update()
{
	Map *map;
	List *list;

	force_update(gamewin);
	for_all_maps(map) {
		force_update(map->window);
	}
	for_all_lists(list) {
		force_update(list->window);
	}
	/* (etc) */
}

beep()
{
	SysBeep(20);
}

#ifdef THINK_C
/* Lifted directly from Think C sources, should replace eventually. */
/* unsigned char * */
c2p(register const char *s, register char *t)
{
	asm {
		movea.l	t,a0			;  A0 ==> start of p-string
		st		d1				;  D1.B = length
@1		move.b	d0,(t)+
		addq.b	#1,d1
		move.b	(s)+,d0
		bne.s	@1
		move.b	d1,(a0)
		move.l	a0,d0			;  return p-string
	}
}
#else
unsigned char *
__c2p(register const char *s, register char *t)
{
}
#endif

/* Shut a single game display. */

close_display(side)
Side *side;
{
	dside->ui->active = FALSE;
	/* And close some windows? */
	Dprintf("Mac display closed.\n");
	/* should attempt to release all memory. */
}

dump_view(side)
Side *side;
{
}

/* (should get name from a preference) */

char *
checkpoint_name()
{
	return "Checkpoint";
}

update_everything()
{
	if (active_display(dside)) {
		force_overall_update();
	}
}

setfiletype(char *name)
{
#ifdef THINK_C
	FileParam pb;
	Str255 tmpstr;
	
	c2p(name, tmpstr);
	pb.ioNamePtr = tmpstr;
	pb.ioVRefNum = 0;
	pb.ioFVersNum = 0;
	pb.ioFDirIndex = 0;
	if (PBGetFInfoSync(&pb) == noErr) {
		pb.ioFlFndrInfo.fdType = 'TEXT';
		pb.ioFlFndrInfo.fdCreator = 'XCNQ';
		PBSetFInfoSync(&pb);
	}
#endif
}

/* Returns true if we can connect to another game. */

can_connect()
{
	return TRUE;
}