/* Copyright (c) 1992  Stanley T. Shebs. */
/* This program may be used, copied, modified, and redistributed freely */
/* for noncommercial purposes, so long as this notice remains intact. */

/* Picture drawing for the Mac interface. */

#include "conq.h"
#include "mac.h"

/* The following magical arrays set all the sizes at each mag. */

/* This is the basic cell size. */

int mags[] = { 1, 2, 4, 8, 16, 32, 64, 128 };

/* These give the total dimensions of each hex cell, plus the vertical
   distance center-to-center.  This is all carefully calculated to make
   the cells overlap perfectly at each different magnification, assuming
   that the icons have the right shape and size. */

int hws[] = { 1, 2, 4, 8, 20, 40, 80, 160 };
int hhs[] = { 1, 2, 4, 8, 22, 44, 88, 176 };
int hcs[] = { 1, 2, 4, 8, 17, 34, 68, 136 };

int gridnw[] = { 0, 0, 0, 0, 0, 1, 1, 1 };
int gridse[] = { 0, 0, 0, 0, 1, 1, 1, 2 };

/* The sizes of the unit subcells.  This is available drawing area, exact
   unit icon sizes depends on what's available. */

int uhs[] = { 1, 2, 3, 8, 16, 32, 64, 128 };
int uws[] = { 1, 2, 3, 8, 16, 32, 64, 128 };

/* Adjust uh down to fix exactly within a hex. */

int iuhs[] = { 1, 2, 2, 8, 14, 28, 56, 112 };
int iuws[] = { 1, 2, 2, 8, 16, 32, 64, 128 };

/* Widths of borders and connections (0 implies don't draw at all). */

int bwid[NUMPOWERS]  = { 0, 0, 1, 1, 3, 5, 7, 9 };

int bwid2[NUMPOWERS] = { 0, 0, 1, 1, 2, 3, 4, 5 };

int cwid[NUMPOWERS] = { 0, 0, 1, 1, 3, 5, 7, 9 };

/* Coordinates of the hex borders. */
/* Note that array has extra entry so don't need to wrap index. */

int bsx[NUMPOWERS][7] = {
	{ 0 },
	{ 0 },
	{  2,   4,   4,  2,  0,  0,  2 },
	{  4,   8,   8,  4,  0,  0,  4 },
	{ 10,  20,  20, 10,  0,  0, 10 },
	{ 20,  40,  40, 20,  0,  0, 20 },
	{ 40,  80,  80, 40,  0,  0, 40 },
	{ 80, 160, 160, 80,  0,  0, 80 }
};
int bsy[NUMPOWERS][7] = {
	{ 0 },
	{ 0 },
	{  0,  0,  4,  4,  4,  0,  0 },
	{  0,  0,  8,  8,  8,  0,  0 },
	{  0,  4, 18, 22, 18,  4,  0 },
	{  0,  8, 36, 44, 36,  8,  0 },
	{  0, 16, 72, 88, 72, 16,  0 },
	{  0, 32, 144, 176, 144, 32,  0 }
};

/* Coords of middles of each hex border (half a connection, basically). */
 
int lsx[NUMPOWERS][6] = {
	{ 0 },
	{ 0 },
	{  1,  2,   1,  -1,  -2,  -1 },
	{  2,  4,   2,  -2,  -4,  -2 },
	{  5, 10,   5,  -5, -10,  -5 },
	{ 10, 20,  10, -10, -20, -10 },
	{ 20, 40,  20, -20, -40, -20 },
	{ 40, 80,  36, -36, -76, -36 }
};
int lsy[NUMPOWERS][6] = {
	{ 0 },
	{ 0 },
	{  -2,  0,   2,   2,   0,  -2 },
	{  -4,  0,   4,   4,   0,  -4 },
	{  -9,  0,   9,   9,   0,  -9 },
	{ -18,  0,  18,  18,   0, -18 },
	{ -36,  0,  36,  36,   0, -36 },
	{ -68,  0,  68,  68,   0, -68 }
};

PolyHandle polygons[NUMPOWERS];
int lastpolyx[NUMPOWERS], lastpolyy[NUMPOWERS];
PolyHandle gridpolygons[NUMPOWERS];
int lastgridpolyx[NUMPOWERS], lastgridpolyy[NUMPOWERS];

Image **bestterrainimages = NULL;

/* Draw an image of the given type of unit in the given rectangle, possibly adding
   an emblem if requested. */

draw_unit_image(win, sx, sy, sw, sh, u, e, mod)
WindowPtr win;
int sx, sy, sw, sh, u, e, mod;
{
	int ew, eh;
	Rect srcrect, imagerect;
	RgnHandle tmprgn;
	BitMap bm, *winbits;
	Image *uimg;

	imagerect = win->portRect;
	imagerect.top += sy;  imagerect.left += sx;
	imagerect.bottom = imagerect.top + sh;  imagerect.right = imagerect.left + sw;
	uimg = best_image(&(uimages[u]), sw, sh);
	/* There should always be *some* image to display. */
	if (uimg) {
		winbits = &(((GrafPtr) win)->portBits);
		if (uimg->monopict != nil) {
			DrawPicture(uimg->monopict, &imagerect);
		} else if (uimg->colricon != nil) {
			PlotCIcon(&imagerect, (CIconHandle) uimg->colricon);
		} else if (uimg->monoicon != nil) {
			SetRect(&srcrect, 0, 0, 32, 32);
			bm.rowBytes = 4;
			bm.bounds = srcrect;
			if (uimg->maskicon != nil) {
				bm.baseAddr = *(uimg->maskicon);
				CopyBits(&bm, winbits, &srcrect, &imagerect, srcBic, nil);
			} else {
				/* Draw unit bbox as default mask. (maybe shrink a little??) */
				FillRect(&imagerect, QD(white));
			}
			bm.baseAddr = *(uimg->monoicon);
			CopyBits(&bm, winbits, &srcrect, &imagerect, srcOr, nil);
		} else if (uimg->monosicn != nil) {
			SetRect(&srcrect, 0, 0, 16, 16);
			bm.rowBytes = 2;
			bm.bounds = srcrect;
			if (uimg->masksicn != nil) {
				bm.baseAddr = *(uimg->masksicn);
				CopyBits(&bm, winbits, &srcrect, &imagerect, srcBic, nil);
			} else {
				/* Draw unit bbox as default mask. (maybe shrink a little??) */
				FillRect(&imagerect, QD(white));
			}
			bm.baseAddr = *(uimg->monosicn);
			CopyBits(&bm, winbits, &srcrect, &imagerect, srcOr, nil);
		} else {
			run_warning("Image with no content?");
		}
	} else {
		FillRect(&imagerect, QD(black));
	}
	if (mod != 0) {
		gray_out_rect(&imagerect);
	}
	/* Now draw a side emblem if asked for. */
	if (between(0, e, numsides)) {
		if (1 /* !emblem_is_builtin() */) {
			if (0 /* emblem loc specified */) {
			} else {
				ew = min(sw, max(8, sw / 4));  eh = min(sh, max(8, sh / 4));
			}
			draw_side_emblem(win, sx + sw - ew, sy, ew, eh, side_n(e));
		}
	}
}

/* Draw a given side's emblem. Uses the current GrafPort. */

draw_side_emblem(win, ex, ey, ew, eh, side2)
WindowPtr win;
int ex, ey, ew, eh;
Side *side2;
{
	int s2 = side_number(side2);
	Rect srcrect, imagerect = win->portRect;
	RgnHandle tmprgn;
	BitMap bm, *winbits;
	Image *eimg;

	eimg = best_image(&(eimages[s2]), ew, eh);
	imagerect.top += ey;  imagerect.left += ex;
	imagerect.bottom = imagerect.top + eh;  imagerect.right = imagerect.left + ew;
	/* If an image is available, display it, otherwise do nothing. */
	if (eimg) {
		winbits = &(((GrafPtr) win)->portBits);
		if (eimg->monopict != nil) {
			DrawPicture(eimg->monopict, &imagerect);
		} else if (eimg->colricon != nil) {
			PlotCIcon(&imagerect, (CIconHandle) eimg->colricon);
		} else if (eimg->monoicon != nil) {
			SetRect(&srcrect, 0, 0, 32, 32);
			bm.rowBytes = 4;
			bm.bounds = srcrect;
			if (eimg->maskicon != nil) {
				bm.baseAddr = *(eimg->maskicon);
				CopyBits(&bm, winbits, &srcrect, &imagerect, srcBic, nil);
			} else {
				/* Use emblem bbox as default mask. */
				FillRect(&imagerect, QD(white));
			}
			bm.baseAddr = *(eimg->monoicon);
			CopyBits(&bm, winbits, &srcrect, &imagerect, srcOr, nil);
		} else if (eimg->monosicn != nil) {
			SetRect(&srcrect, 0, 0, 16, 16);
			bm.rowBytes = 2;
			bm.bounds = srcrect;
			if (eimg->masksicn != nil) {
				bm.baseAddr = *(eimg->masksicn);
				CopyBits(&bm, winbits, &srcrect, &imagerect, srcBic, nil);
			} else {
				/* Use emblem bbox as default mask. */
				FillRect(&imagerect, QD(white));
			}
			bm.baseAddr = *(eimg->monosicn);
			CopyBits(&bm, winbits, &srcrect, &imagerect, srcOr, nil);
		} else {
			run_warning("Image with no content?");
		}
	}
}

draw_hex_block(sx, sy, n, power, t, t2)
int sx, sy, n, power, t, t2;
{
	Rect rect;
	Image *timg;
	RGBColor hexcolor, oldcolor;				

	rect.left = sx;  rect.top = sy;
	rect.right = rect.left + n * hws[power];  rect.bottom = rect.top + hcs[power];
	if (bestterrainimages == NULL) {
		calc_best_terrain_images();
	}
	timg = bestterrainimages[power * numttypes + t];
	if (timg) {
		if (hasColorQD) {
			if (timg->colrpat != nil) {
				FillCRect(&rect, timg->colrpat);
			} else if (tcolors[t] != NULL) {
				hexcolor.red   = (tcolors[t]->r) << 8;
				hexcolor.green = (tcolors[t]->g) << 8;
				hexcolor.blue  = (tcolors[t]->b) << 8;
				RGBForeColor(&hexcolor);
				PaintRect(&rect);
				/* Restore the previous color. */
				oldcolor.red = oldcolor.green = oldcolor.blue = 0;
				RGBForeColor(&oldcolor);
			} else if (timg->patdefined) {
#ifdef THINK_C
				FillRect(&rect, (Pattern *) &(timg->pat));
#endif
			}
		} else {
#ifdef THINK_C
			FillRect(&rect, (Pattern *) &(timg->pat));
#endif
		}
		if (t2 >= 0) {
			PenPat(QD(gray));
/*			PenMode(patBic);  */
			PenMode(notPatOr);
			PaintRect(&rect);
			PenNormal();
		}
	}
}

calc_best_terrain_images()
{
	int p, t;

	bestterrainimages = (Image **) xmalloc(NUMPOWERS * numttypes * sizeof(Image *));
	for_all_terrain_types(t) {
		for (p = 0; p < NUMPOWERS; ++p) {
			bestterrainimages[p * numttypes + t] = 
				best_image(&(timages[t]), hws[p], hcs[p]);
		}
	}
}

/* Do the grody work of drawing very large polygons accurately.  Do this by calculating
   the hex polygon only once (when first needed), when offsetting each time to draw. */

draw_hex_polygon(sx, sy, power, dogrid, t, t2)
int sx, sy, power, dogrid, t, t2;
{
	int hw = hws[power], hh = hhs[power], delt = (hhs[power] - hcs[power]);
	int nw = 0, se = 0;
	PolyHandle poly;
	Image *timg;
	RGBColor hexcolor, oldcolor;

	if (dogrid) {
		poly = gridpolygons[power];
	} else {
		poly = polygons[power];
	}
	if (poly == nil) {
		if (dogrid) { nw = gridnw[power];  se = gridse[power]; }
		poly = OpenPoly();
		if (hw != 20) {		
			MoveTo(hw / 2, nw);
			LineTo(hw - se, delt);
			LineTo(hw - se, hh - delt);
			LineTo(hw / 2, hh - se);
			LineTo(0 + nw, hh - delt);
			LineTo(0 + nw, delt);
			LineTo(hw / 2, nw);
		} else {
			MoveTo(10, 0);
			LineTo(20 - se, 5);
			LineTo(20 - se, 17);
			LineTo(10, 22 - se);
			LineTo(0, 17);
			LineTo(0, 5);
			LineTo(10, 0);
		}
		ClosePoly();
		if (dogrid) {
			gridpolygons[power] = poly;
			lastgridpolyx[power] = lastgridpolyy[power] = 0;
		} else {
			polygons[power] = poly;
			lastpolyx[power] = lastpolyy[power] = 0;
		}
	}
	if (dogrid) {
		OffsetPoly(poly, sx - lastgridpolyx[power], sy - lastgridpolyy[power]);
		lastgridpolyx[power] = sx;  lastgridpolyy[power] = sy;
	} else {
		OffsetPoly(poly, sx - lastpolyx[power], sy - lastpolyy[power]);
		lastpolyx[power] = sx;  lastpolyy[power] = sy;
	}
	if (bestterrainimages == NULL) {
		calc_best_terrain_images();
	}
	timg = bestterrainimages[power * numttypes + t];
	if (timg) {
		if (hasColorQD) {
			if (timg->colrpat != nil) {
				FillCPoly(poly, timg->colrpat);
			} else if (tcolors[t] != NULL) {
				hexcolor.red   = (tcolors[t]->r) << 8;
				hexcolor.green = (tcolors[t]->g) << 8;
				hexcolor.blue  = (tcolors[t]->b) << 8;
				RGBForeColor(&hexcolor);
				PaintPoly(poly);
				/* Restore the previous color. */
				oldcolor.red = oldcolor.green = oldcolor.blue = 0;
				RGBForeColor(&oldcolor);
			} else if (timg->patdefined) {
				/* Fall back on the b/w pattern. */
#ifdef THINK_C
				FillPoly(poly, (Pattern *) &(timg->pat));
#endif
			}
		} else {
#ifdef THINK_C
			FillPoly(poly, (Pattern *) &(timg->pat));
#endif
		}
		if (t2 >= 0) {
			PenPat(QD(gray));
/*			PenMode(patBic);  */
			PenMode(notPatOr);
			PaintPoly(poly);
			PenNormal();
		}
	}
}

/* Draw a single border on the given edge of the given hex. */

draw_border_line(sx, sy, dir, power, t)
int sx, sy, dir, power, t;
{
	int wid = bwid[power];
	Image *timg;

	if (wid == 0) return;
	MoveTo(sx + bsx[power][dir] - wid / 2, sy + bsy[power][dir] - wid / 2);
	PenSize(wid, wid);
	if (is_terrain_type(t)) {
		/* Decide on the line color/pattern to use. */
		timg = best_image(&(timages[t]), wid, wid);
		if (timg) {
			if (hasColorQD) {
				if (timg->colrpat) {
					PenPixPat(timg->colrpat);
				} else {
#ifdef THINK_C
					PenPat((Pattern *) &(timg->pat));
#endif
				}
			} else {
#ifdef THINK_C
				PenPat((Pattern *) &(timg->pat));
#endif
}
		} else {
			PenPat(QD(dkGray));
		}
	} else {
		/* Not a terrain type, is country border or some such.  Draw in black. */
		PenPat(QD(black));
		PenSize(0 - t, 0 - t);
	}
	LineTo(sx + bsx[power][dir+1] - wid / 2, sy + bsy[power][dir+1] - wid / 2);
	PenNormal();
}

/* Draw a single piece of line representing connection terrain. */

draw_connection_line_half(sx, sy, dir, power, t)
int sx, sy, dir, power, t;
{
	int wid = cwid[power];
	Image *timg;

	if (wid == 0) return;
	MoveTo(sx + hws[power] / 2 - wid / 2, sy + hhs[power] / 2 - wid / 2);
	PenSize(wid, wid);
	timg = best_image(&(timages[t]), wid, wid);
	if (timg) {
		if (hasColorQD) {
			if (timg->colrpat) {
				PenPixPat(timg->colrpat);
			} else {
#ifdef THINK_C
				PenPat((Pattern *) &(timg->pat));
#endif
			}
		} else {
#ifdef THINK_C
			PenPat((Pattern *) &(timg->pat));
#endif
		}
	} else {
		PenPat(QD(gray));
	}
	Line(lsx[power][dir], lsy[power][dir]);
	PenNormal();
}

/* Draw a single country border at the given position. */

draw_country_border_line(sx, sy, dir, power)
int sx, sy, dir, power;
{
	int wid = bwid2[power], wid2;

	if (wid == 0) return;
	wid2 = wid / 2;
	PenSize(wid, wid);
	PenPat(QD(black));
	wid = 0;
	MoveTo(sx + bsx[power][dir] - wid, sy + bsy[power][dir] - wid);
	LineTo(sx + bsx[power][dir+1] - wid, sy + bsy[power][dir+1] - wid);
	PenNormal();
}

draw_legend_text(sx, sy, sh, legend, just)
int sx, sy, sh, just;
char *legend;
{
	int strwid, strleft;
	Rect maskrect;
	FontInfo fontinfo;
	Str255 tmpstr;
	
	strwid = TextWidth(legend, 0, strlen(legend));
	if (just < 0) {
		strleft = sx;
	} else if (just > 0) {
		strleft = sx - strwid;
	} else {
		strleft = sx - strwid / 2;
	}
	MoveTo(strleft, sy);
	TextSize(min(max(5, sh), 10));
	if (0) {
		/* Make it readable against a noisy background. */
		TextFace(bold|outline);
		TextMode(srcBic);
	} else {
		/* This makes a big white box for name, less attractive but easier to read. */
		GetFontInfo(&fontinfo);
		maskrect.top = sy - fontinfo.ascent - 1;  /* this seems to be too much? */
		maskrect.top += 3;
		maskrect.left = strleft - 1;
		maskrect.bottom = sy + fontinfo.descent + 1;
		maskrect.right = maskrect.left + strwid + 2;
		FillRect(&maskrect, QD(white));
	}
	c2p(legend, tmpstr);
	DrawString(tmpstr);
}

draw_blast_image(win, sx, sy, sw, sh, power, blasttype)
WindowPtr win;
int sx, sy, sw, sh, blasttype;
{
	long startcount;
	Rect tmprect;
	Handle sound;

	SetRect(&tmprect, sx, sy, sx + sw, sy + sh);
	if (power >= 4) {
		/* Instead should save image under here, then draw blast */
		InvertRect(&tmprect);
	} else {
		InvertRect(&tmprect);
	}
	startcount = TickCount();
    sound = GetResource('snd ', 128);
    if (sound != nil) {
        SndPlay(nil, sound, false);
        ReleaseResource(sound);
    }
    /* Delay 1/2 second in all (should relinquish cpu tho) */
	while ((TickCount() - startcount) < (blasttype > 0 ? 30 : 10));
	if (power >= 4) {
		/* Instead should restore image under here */
		InvertRect(&tmprect);
	} else {
		InvertRect(&tmprect);
    }
}

picture_width(pichandle)
PicHandle pichandle;
{
	return ((*pichandle)->picFrame.right - (*pichandle)->picFrame.left);
}

picture_height(pichandle)
PicHandle pichandle;
{
	return ((*pichandle)->picFrame.bottom - (*pichandle)->picFrame.top);
}

/* Generic sicn drawer. */

plot_sicn(win, sx, sy, sicnhandle)
WindowPtr win;
int sx, sy;
Handle sicnhandle;
{
	Rect srcrect, imagerect;
	RgnHandle tmprgn;
	BitMap bm, *winbits;

	if (sicnhandle == nil) return;
	imagerect = win->portRect;
	imagerect.left += sx;  imagerect.top += sy;
	imagerect.right = imagerect.left + 16;  imagerect.bottom = imagerect.top + 16;
	winbits = &(((GrafPtr) win)->portBits);
	SetRect(&srcrect, 0, 0, 16, 16);
	bm.rowBytes = 2;
	bm.bounds = srcrect;
	EraseRect(&imagerect);
	bm.baseAddr = *(sicnhandle);
	CopyBits(&bm, winbits, &srcrect, &imagerect, srcOr, nil);
}

/* Given a rectangle, make half of its pixels white. */

gray_out_rect(rectptr)
Rect *rectptr;
{
	PenPat(QD(gray));
	PenMode(patBic);
	PaintRect(rectptr);
	PenNormal();
}

/* Try to find the best of multiple images for the given bounding box. */
/* Don't return anything that won't fit in min space. */

Image *
best_image(imf, w, h)
ImageFamily *imf;
int w, h;
{
	int i;
	Image *img, *best = NULL, *fallback = NULL;
	
	for (i = 0; i < imf->numsizes; ++i) {
		img = &(imf->images[i]);
		/* Exact matches need no further searching. */
		if (w == img->w && h == img->h) {
			return img;
		} else if (between(img->minw, w, img->maxw) && between(img->minh, h, img->maxh)) {
			/* Image is plausible - go for the largest one that will fit. */
			if (!best || (img->w > best->w && img->h > best->h)) {
				best = img;
			}
		} else if (w >= img->minw && h >= img->minh) {
			/* Image is not really big enough, but keep as a fallback. */
			if (!fallback || (img->w > fallback->w && img->h > fallback->h)) {
				fallback = img;
			}
		}
	}
	return (best ? best : fallback);
}
