/* Copyright (c) 1992  Stanley T. Shebs. */
/* This program may be used, copied, modified, and redistributed freely */
/* for noncommercial purposes, so long as this notice remains intact. */

/* Initialization of the Mac interface. */

#include "conq.h"
#include "mac.h"

#define MAXCHECKBOXES 10
#define MAXSLIDERS 4

#define toggle_checkbox(ih) \
  SetCtlValue((ControlHandle) (ih), !GetCtlValue((ControlHandle) (ih)))

/* (not quite right - come up with a better test) */

#define game_already_loaded() (numutypes > 0)

#define t_color_defined(t) (hasColorQD && tcolors != NULL && tcolors[t] != NULL)

#define p2c(pstr,buf)  \
  strncpy(buf, pstr+1, pstr[0]);  \
  /* Make sure this name is terminated properly. */  \
  buf[pstr[0]] = '\0';

extern Assign *assignments;
extern int unequalstrengths;
extern int foundresourcefile;
extern int allbedesigners;

char *copy_string();

char *firstgamename = "intro"; /* into a special str resource?? */

int varyseen;
int varyallseen;
int varyworld;
int anyvariants;

/* This is the dialog used to select a game to play. */

WindowPtr newgamewin = nil;

ListHandle newgamelist = nil;

TEHandle newgametext = nil;

PicHandle newgamepicture = nil;

Module **possiblegames = NULL;

int numgames = 0;

Module *selectedgame = NULL;

/* This is the dialog used to set the size and shape of the world. */

DialogPtr worldshapewin = nil;

/* These are the values that will be used to set world geometry. */

long newcircumference;
long newwidth;
long newheight;
long newlatitude;
long newlongitude;
long newxwrap;

int newseen;
int newseeall;

WindowPtr variantswin = nil;

int numcheckboxes;
int numsliders;

Obj *checkboxes[MAXCHECKBOXES];
char *checkboxnames[MAXCHECKBOXES];
Obj *sliders[MAXSLIDERS];
char *slidernames[MAXSLIDERS];

WindowPtr playersetupwin = nil;

int playerh = 20;
int playerbaseline = 14;

int selectedplayer = -1;

#define selected_a_player() (between(0, selectedplayer, numsides - 1))

#define selected_player() (assignments[selectedplayer].player)

#define selected_side() (assignments[selectedplayer].side)

ImageFamily *uimages = NULL;
ImageFamily *timages = NULL;
ImageFamily *eimages = NULL;

ImageColor **tcolors = NULL;

CursHandle readprogressors[NUMcParens];
CursHandle progressors[NUMcSynth];
CursHandle movecursors[NUMcMoves];
CursHandle allmovecursor;
CursHandle opencrosscursor;
CursHandle watchcursor;

/* This value is the current progress cursor. */

int curcurs = 0;

/* The initialization progress dialog. */

DialogPtr progresswin = nil;

/* The current value of the percent progress. */

int progress;

/* The previous value of the percent progress. */

int lastprogress;

/* Set up the various cursors we'll be using. */

init_cursors()
{
	int i;

	/* Get all the cursors used to indicate reader progress. */
	for (i = 0; i < NUMcParens; ++i) {
		readprogressors[i] = GetCursor(cParens1 + i);
	}
	/* Get all the cursors used to indicate synthesis progress. */
	for (i = 0; i < NUMcSynth; ++i) {
		progressors[i] = GetCursor(cSynth1 + i);
	}
	/* Get all the cursors indicating the move direction. */
	for (i = 0; i < NUMcMoves; ++i) {
		movecursors[i] = GetCursor(cMove1 + i);
	}
	allmovecursor = GetCursor(cAllMove);
	opencrosscursor = GetCursor(cOpenCross);
	watchcursor = GetCursor(watchCursor);
	/* Designer-related cursors will be done later, if needed at all. */
}

/* Dialog user item callback that displays the list of possible games. */

pascal void
draw_game_list(win, ditem)
WindowPtr win;
short ditem;
{
	short itemtype;  Handle itemhandle;  Rect itemrect;

	GetDItem(newgamewin, diNewGameList, &itemtype, &itemhandle, &itemrect);
	/* Draw the list of available games. */
	LUpdate(newgamewin->visRgn, newgamelist);
	/* Frame it nicely. */
	--itemrect.left;
	FrameRect(&itemrect);
}

/* Dialog user item callback to display the game description. */

pascal void
draw_game_blurb(win, ditem)
WindowPtr win;
short ditem;
{
	GrafPtr oldport;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	GetDItem(newgamewin, diNewGameBlurb, &itemtype, &itemhandle, &itemrect);
	/* Make sure the text is up-to-date. */
	GetPort(&oldport);
	SetPort(newgamewin);
	TEUpdate(&itemrect, newgametext);	
	SetPort(oldport);
	/* Frame it. */
	InsetRect(&itemrect, -1, -1);
	FrameRect(&itemrect);
}

/* Dialog user item callback to display the game picture, if available. */

pascal void
draw_game_picture(win, ditem)
WindowPtr win;
short ditem;
{
	short itemtype;  Handle itemhandle;  Rect itemrect;

	GetDItem(newgamewin, diNewGamePicture, &itemtype, &itemhandle, &itemrect);
	EraseRect(&itemrect);
#ifdef DESIGNERS
	/* A low-impact feedback that this game will launch directly into designing. */
	if (allbedesigners) {
		InsetRect(&itemrect, -2, -2);
		FillRect(&itemrect, QD(ltGray));
		InsetRect(&itemrect, 2, 2);
	}
#endif /* DESIGNERS */
	if (newgamepicture != nil) {
		center_pict(newgamepicture, &itemrect);
		DrawPicture(newgamepicture, &itemrect);
	}
}

center_pict(picture, rect)
PicHandle picture;
Rect *rect;
{
	Rect initrect, pictrect;

	initrect = *rect;
	pictrect = (*picture)->picFrame;
	rect->left += (initrect.right - initrect.left) / 2 - (pictrect.right - pictrect.left) / 2;
	rect->top += (initrect.bottom - initrect.top) / 2 - (pictrect.bottom - pictrect.top) / 2;
	rect->right = rect->left + (pictrect.right - pictrect.left);
	rect->bottom = rect->top + (pictrect.bottom - pictrect.top);
}

Module *
module_from_cell(cell)
Cell cell;
{
	return (cell.v < numgames ? possiblegames[cell.v] : NULL);
}

/* This filter for the new game dialog just handles the game list, updating the
   other items to reflect the currently selected game. */

pascal short
filter_new_game(dialog, evt, itemhit)
DialogPtr dialog;
EventRecord *evt;
short *itemhit;
{
	GrafPtr oldport;
	Point pt;
	short ditem;
	char ch;

	/* No list tricks to do if game already loaded. */
	if (game_already_loaded()) return FALSE;
	/* Look for the right kind of event. */
	switch (evt->what) {
		case mouseDown:
			GetPort(&oldport);
			SetPort(newgamewin);
			pt = evt->where;
			GlobalToLocal(&pt);
			ditem = FindDItem(newgamewin, pt) + 1;
			if (ditem == diNewGameList) {
				/* Loop around in list handling. */
				LClick(pt, evt->modifiers, newgamelist);
				/* We're finished clicking, show the results. */
				select_game();
				display_selected_game();
				SetPort(oldport);
				*itemhit = diNewGameList;
				return TRUE;
			} else {
				SetPort(oldport);
				return FALSE;
			}
			break;
		case keyDown:
			ch = evt->message & charCodeMask;
			if (ch == 3 || ch == 13) {
				if (selectedgame) {
					*itemhit = diNewGameOK;
					return TRUE;
				}
			}
#ifdef DESIGNERS
			if (ch == 'd') {
				allbedesigners = !allbedesigners;
			}
#endif
			break;
	}
	return FALSE;
}

/* Given a selected game, display assorted info about it - basically a preview so that
   prospective players can see what they're getting into.  If no game has been selected,
   then this routine clears the displays. */

display_selected_game()
{
	short itemtype;  Handle itemhandle;  Rect itemrect;
	char *desc, buf[BUFSIZE];
	Str255 tmpstr;
	PicHandle oldnewgamepicture;

	DGprintf("display selected game %s\n", module_desig(selectedgame));
	/* Set the blurb for the selected game. */
	GetDItem(newgamewin, diNewGameBlurb, &itemtype, &itemhandle, &itemrect);
	TESetSelect(0, 100000, newgametext);
	TECut(newgametext);
	if (selectedgame /* && selectedgame->complete */) {
		if (selectedgame->blurb) {
			desc = selectedgame->blurb;
		} else {
			desc = "??? experimental ???";
		}
	} else {
		desc = "";
	}
	TESetText(desc, strlen(desc), newgametext);
	TEUpdate(&itemrect, newgametext);
	/* Load a picture if one can be found. */
	/* (should look for picture explicitly named in a game-module slot first) */
	oldnewgamepicture = newgamepicture;
	newgamepicture = nil;
	if (selectedgame) {
		sprintf(spbuf, "%s game", selectedgame->name);
		c2p(spbuf, tmpstr);
		newgamepicture = (PicHandle) GetNamedResource('PICT', tmpstr);
		if (newgamepicture == nil) {
			sprintf(spbuf, "%s", selectedgame->name);
			c2p(spbuf, tmpstr);
			newgamepicture = (PicHandle) GetNamedResource('PICT', tmpstr);
		}
	} else {
		/* Nothing to display - how boring! */
	}
	/* Gray out the OK button if no game selected. */
	GetDItem(newgamewin, diNewGameOK, &itemtype, &itemhandle, &itemrect);
	HiliteControl((ControlHandle) itemhandle, (selectedgame ? 0 : 255));
	/* We have to force redraw to get the picture item updated. */
	if (oldnewgamepicture != newgamepicture) DrawDialog(newgamewin);
}

/* This is a modal dialog from which the user selects a game and sets options. */

new_game_dialog()
{
	int done = FALSE;
	short ditem, i;
	Point pnt, cellsize;
	Cell tmpcell;
	Rect listrect, destrect, viewrect;
	Module *module;
	char *gamename;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	collect_possible_games();
	if (newgamewin == nil) {
		newgamewin = GetNewDialog(dNewGame, NULL, (DialogPtr) -1);
		SetPort(newgamewin);
		TextFont(newYork);
/*		TextSize(10); */
		/* Set up the user item that lists games. */
		GetDItem(newgamewin, diNewGameList, &itemtype, &itemhandle, &itemrect);
		SetDItem(newgamewin, diNewGameList, itemtype, (Handle) draw_game_list, &itemrect);
		SetPt(&cellsize, 0, 0);
		listrect.top = 0;  listrect.left = 0;
		listrect.bottom = 0;  listrect.right = 1;
		itemrect.top += 1;
		itemrect.bottom -= 1;  itemrect.right -= sbarwid + 1;
		/* Create the list of games itself and fill it in. */
		newgamelist = LNew(&itemrect, &listrect, cellsize, 0, newgamewin, 0,0,0,TRUE);
		SetPt(&tmpcell, 0, 0);
		for (i = 0; i < numgames; ++i) {
			module = possiblegames[i];
			gamename = module->title;
			if (gamename == NULL) gamename = module->name;
			if (gamename == NULL) gamename = "???";
			LAddRow(1, tmpcell.v, newgamelist);
			LSetCell(gamename, strlen(gamename), tmpcell, newgamelist);
			++tmpcell.v;
		}
		GetDItem(newgamewin, diNewGameBlurb, &itemtype, &itemhandle, &itemrect);
		SetDItem(newgamewin, diNewGameBlurb, itemtype, (Handle) draw_game_blurb, &itemrect);
		destrect = itemrect;
		viewrect = itemrect;
/*		TextSize(10);  */
		newgametext = TENew(&destrect, &viewrect);
		GetDItem(newgamewin, diNewGamePicture, &itemtype, &itemhandle, &itemrect);
		SetDItem(newgamewin, diNewGamePicture, itemtype, (Handle) draw_game_picture, &itemrect);
		newgamepicture = nil;
	}
	update_new_game_list();
	SetPt(&tmpcell, 0, 0);
	LSetSelect(TRUE, tmpcell, newgamelist);  /* (do this only for new/changed list?) */
	select_game();
	display_selected_game();
	SetCursor(&QD(arrow));
	ShowWindow(newgamewin);
	LDoDraw(1, newgamelist);
	/* Loop around here until the player picks a game to play, or cancels. */
	while (!done) {
		draw_default_button(newgamewin, diNewGameOK);
		ModalDialog((ModalFilterProcPtr) filter_new_game, &ditem);
		switch (ditem) {
			case diNewGameOK:
				/* Make this dialog disappear, startup may take a long time. */
				HideWindow(newgamewin);
				if (start_new_game()) {
					done = TRUE;
				} else {
					/* If we failed to start the new game, just go around again. */
					ShowWindow(newgamewin);
					SelectWindow(newgamewin);
				}
				break;
			case diNewGameCancel:
				done = TRUE;
				break;
			default:
				break;
		}
	}
	/* We're done, OK to throw away the dialog. */
	DisposDialog(newgamewin);
	newgamewin = nil;
	/* (should release TEs and lists also) */
}

/* The comparison function for the game list puts un-formally-named modules at the end,
   plus the default sorting puts initial-lowercased names after uppercased ones. */

module_name_compare(mp1, mp2)
Module **mp1, **mp2;
{
	if ((*mp1)->title == NULL) {
		if ((*mp2)->title == NULL) {
			/* (this assumes we have no entirely unnamed modules) */
			return strcmp((*mp1)->name, (*mp2)->name);
		} else {
			return 1;
		}
	} else {
		if ((*mp2)->title == NULL) {
			return (-1);
		} else {
			return strcmp((*mp1)->title, (*mp2)->title);
		}
	}
}

/* This routine finds and lists all the games that should be listed as choices for
   the user. */

collect_possible_games()
{
	int numresources, i;
	Handle modulehandle;
	short moduleid;
	char *modulename = NULL, *modulecontents = NULL;
	ResType restype;
	Str255 resname;
	Module *module;

	if (numgames == 0 && !game_already_loaded()) {
		numresources = CountResources('XCgm');
		/* Make enough room to record all the possible games. */
		possiblegames = (Module **) xmalloc((numresources + 1) * sizeof(Module *));
		/* Collect the intro game module and put at head of list. */
		module = get_game_module(firstgamename);
		add_to_possible_games(module);
		/* Pick up game modules that are wired in as resources. */
		for (i = 0; i < numresources; ++i) {
			modulehandle = GetIndResource('XCgm', i + 1);
			/* (should test for resource validity?) */
			if (0 /* size > 0 */) {
				/* set modulecontents from resource */
				modulecontents = NULL;
			}
			/* Try to pick up module name from its resource name, otherwise
			   assume its name in its content. */
			GetResInfo(modulehandle, &moduleid, &restype, &resname);
			if (resname[0] > 0) {
				resname[resname[0]+1] = '\0';
				modulename = copy_string(resname+1);
			} else {
				modulename = NULL;
			}
			module = get_game_module(modulename);
			module->contents = modulecontents;
			add_to_possible_games(module);
		}
#if 0
		/* (eventually add random library files) */
		/* (will have to count them first) */
		printf("Lib is %s\n", xconqlib);
		sprintf(spbuf, "%s:", xconqlib);
		pascalify(spbuf);
		for (i = 0; i < 10; ++i) {
			FInfo info;
			OSErr rslt = GetFInfo(tmpbuf, i, &info);
			
			printf("%d: rslt is %d\n", i, rslt);
			if (rslt == noErr) {
				printf("  %c %c %d %d %d\n",
						info.fdType, info.fdCreator, info.fdFlags,
						info.fdLocation, info.fdFldr);
			}
		}
#endif
		if (numgames > 2) {
			/* Sort all but the first game into alphabetical order by displayed name. */
			qsort(&(possiblegames[1]), numgames - 1, sizeof(Module *), module_name_compare);
		}
	}
}

/* Load a game's description and add it to the list of games. */

add_to_possible_games(module)
Module *module;
{
	int i;

	if (module != NULL) {
		if (load_game_description(module)) {
			/* It might be that the module description supplies the real name,
		  	 and that the module already exists. (work on this) */
		  	/* Don't add duplicate modules. */
			for (i = 0; i < numgames; ++i) {
				if (possiblegames[i] == module) return;
			}
		  	if (numgames < 100) {
				possiblegames[numgames++] = module;
			}
		}
	}
}

/* Select a game from the list of possible games. */

select_game()
{
	Cell tmpcell;

	SetPt(&tmpcell, 0, 0);
	if (game_already_loaded()) {
		/* selectedgame is already set correctly. */
	} else if (LGetSelect(TRUE, &tmpcell, newgamelist)) {
		selectedgame = module_from_cell(tmpcell);
	} else {
		selectedgame = NULL;
	}
}

/* When the new game OK button is hit, this will do the work of getting things started,
   possibly pausing to ask about player/side setup. */

start_new_game()
{
	if (selectedgame == NULL) { /* Just in case... */
		SysBeep(20);
		return FALSE;
	}
	if (!game_already_loaded()) {
		/* Suck in the selected module.  This cannot be undone (yet). */
		load_game_module(selectedgame);
		/* Change cursor back, in case it was different during loading. */
		SetCursor(&QD(arrow));
		/* If the loaded game is not valid, we will get an alert somewhere in here,
		   and possibly bomb out if the player chooses not to continue. */
		check_game_validity();
	}
	return launch_game();
}

/* This routine is for when we end up back in the new game dialog,
   after something has already been loaded. */

/* (could also include any game that has this one as a base module) */

update_new_game_list()
{
	char *gamename;
	Point tmpcell;
	
	if (newgamelist != nil && game_already_loaded()) {
		SetPt(&tmpcell, 0, 0);
		/* Clear out the entire list. */
		LDelRow(0, 0, newgamelist);
		/* Add the loaded game back. */ 
		LAddRow(1, tmpcell.v, newgamelist);
		/* Find a name to paste into the list. */
		gamename = selectedgame->title;
		if (gamename == NULL) gamename = selectedgame->name;
		if (gamename == NULL) gamename = "???";
		/* Set and select the one cell. */
		LSetCell(gamename, strlen(gamename), tmpcell, newgamelist);
		LSetSelect(TRUE, tmpcell, newgamelist);
	}
}


pascal void
draw_variant_slider(win, ditem)
WindowPtr win;
short ditem;
{
	Str255 tmpstr;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	GetDItem(win, ditem, &itemtype, &itemhandle, &itemrect);
	if (between(diVariantsFirstSlider, ditem, diVariantsFirstSlider + numsliders - 1)) {
		FrameRect(&itemrect);
		MoveTo(itemrect.left + 2, itemrect.top + 10);
		c2p(slidernames[ditem - diVariantsFirstSlider], tmpstr);
		DrawString(tmpstr);
	} 
}

/* The variants dialog basically handles all the player-settable options defined
   by a game. */

variants_dialog()
{
	int i, done = FALSE, changed = FALSE;
	Str255 tmpstr;
	short ditem;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	interpret_variants();
	if (!anyvariants) return TRUE;
	if (variantswin == nil) {
		variantswin = GetNewDialog(dVariants, NULL, (DialogPtr) -1L);
		/* Give all the slider app items the same proc. */
		for (i = diVariantsFirstSlider; i < (diVariantsFirstSlider + MAXSLIDERS); ++i) {
			GetDItem(variantswin, i, &itemtype, &itemhandle, &itemrect);
			SetDItem(variantswin, i, itemtype, (Handle) draw_variant_slider, &itemrect);
		}
	}
	/* Display/default the standard variants appropriately. */
	set_variant_item(diVariantsWorldSeen, varyseen, TRUE, newseen);
	set_variant_item(diVariantsSeeAll, varyallseen, TRUE, newseeall);
	set_variant_item(diVariantsWorldSize, varyworld, FALSE, 0);
	/* For each random checkbox used, give it a title and make it displayable,
	   otherwise hide it. */
	for (i = diVariantsFirstCheckBox; i < (diVariantsFirstCheckBox + MAXCHECKBOXES); ++i) {
		GetDItem(variantswin, i, &itemtype, &itemhandle, &itemrect);
		if ((i - diVariantsFirstCheckBox) < numcheckboxes) {
			c2p(checkboxnames[i - diVariantsFirstCheckBox], tmpstr);
			SetCTitle((ControlHandle) itemhandle, tmpstr);
			ShowControl((ControlHandle) itemhandle);
			/* and set its initial state */
		} else {
			HideControl((ControlHandle) itemhandle);
		}
	}
	ShowWindow(variantswin);
	while (!done) {
		draw_default_button(variantswin, diVariantsOK);
		ModalDialog(NULL, &ditem);
		switch (ditem) {
			case diVariantsOK:
				implement_variants();
				changed = TRUE;
				/* Fall through to next case. */
			case diVariantsCancel:
				done = TRUE;
				break;
			case diVariantsWorldSeen:
			case diVariantsSeeAll:
				/* Toggle check boxes. */
				GetDItem(variantswin, ditem, &itemtype, &itemhandle, &itemrect);
				toggle_checkbox(itemhandle);
				break;
			case diVariantsWorldSize:
				/* Fire up a separate dialog for world geometry. */
				if (world_shape_dialog()) {
/*					display_world_shape_text(); */
				}
				break;
			default:
				/* Handle all the checkboxes similarly. */
				if (between(diVariantsFirstCheckBox, ditem, diVariantsFirstCheckBox+MAXCHECKBOXES-1)) {
					GetDItem(variantswin, ditem, &itemtype, &itemhandle, &itemrect);
					toggle_checkbox(itemhandle);
				}
				break;
		}
	}
	DisposDialog(variantswin);
	variantswin = nil;
	return changed;
}

set_variant_item(di, flag, flag2, val)
int di;
int flag, flag2, val;
{
	short itemtype;  Handle itemhandle;  Rect itemrect;

	GetDItem(variantswin, di, &itemtype, &itemhandle, &itemrect);
	if (flag) {
		ShowControl((ControlHandle) itemhandle);
		HiliteControl((ControlHandle) itemhandle, 0);
		if (flag2) {
			SetCtlValue((ControlHandle) itemhandle, val);
		}
	} else {
		HideControl((ControlHandle) itemhandle);
	}
}

/* Go through all the variants and set up appropriate flags. */

interpret_variants()
{
	Obj *vars, *var, *vartype, *vardata, *vartmp;
	char *varname, *vartypename = NULL, buf[BUFSIZE];

	varyseen = varyallseen = varyworld = anyvariants = FALSE;
	if (!selectedgame) return;
	numcheckboxes = 0;
	numsliders = 0;
	for (vars = selectedgame->variants; vars != lispnil; vars = cdr(vars)) {
		anyvariants = TRUE;
		var = car(vars);
		if (symbolp(var)) {
			var = cons(var, lispnil);
		} else if (!consp(var)) {
			/* already warned about */
			continue;
		}
		if (stringp(car(var))) {
			varname = c_string(car(var));
			var = cdr(var);
		} else {
			varname = "Variant xxx";
		}
		vartype = car(var);
		vardata = cdr(var);
		if (symbolp(vartype)) vartypename = c_string(vartype);
		switch (keyword_value(vartypename)) {
			case K_WORLD_SEEN:
				varyseen = TRUE;
				newseen = FALSE;
				if (vardata != lispnil) {
					vartmp = eval(car(vardata));
					if (numberp(vartmp)) {
						newseen = c_number(vartmp);
					}
				}
				break;
			case K_SEE_ALL:
				varyallseen = TRUE;
				newseeall = FALSE;
				if (vardata != lispnil) {
					vartmp = eval(car(vardata));
					if (numberp(vartmp)) {
						newseeall = c_number(vartmp);
					}
				}
				break;
			case K_WORLD_SIZE:
				varyworld = TRUE;
				/* Start with some defaults. */
				newcircumference = 360; newwidth = 60;  newheight = 20;
				newlatitude = 0;  newlongitude = 0;
				break;
			case K_FLAG:
				checkboxnames[numcheckboxes] = varname;
				checkboxes[numcheckboxes] = var;
				++numcheckboxes;
				break;
			case K_NUMBER:
				slidernames[numsliders] = varname;
				sliders[numsliders] = var;
				++numsliders;
				break;
			case K_EVAL:
				checkboxnames[numcheckboxes] = varname;
				checkboxes[numcheckboxes] = var;
				++numcheckboxes;
				break;
			default:
				sprintlisp(buf, car(vars));
				init_warning("unknown variant %s, ignoring", buf);
				break;
		}
	}
}

/* This is where we actually change the state of the game according to the variants. */

implement_variants()
{
	int i;
	Obj *varrest;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	if (varyworld) {
		set_world_circumference(newcircumference, TRUE);
		/* It is critically important that users not be able to reshape already-alloced
		   areas, but do let them know that their request had to be overridden. */
		if (((area.width > 0 && area.width != newwidth)
			|| (area.height > 0 && area.height != newheight))
			&& (1 /* some layers (probably) allocated already */)) {
			init_warning("Area dimensions forced to be %d x %d", area.width, area.height);
		} else {
			set_area_shape(newwidth, newheight, TRUE);
		}
		area.latitude = newlatitude;  area.longitude = newlongitude;
	}
	if (area.width < 1) { set_area_shape(60, 30, TRUE); }
	if (varyseen) {
		GetDItem(variantswin, diVariantsWorldSeen, &itemtype, &itemhandle, &itemrect);
		set_g_terrain_seen(GetCtlValue((ControlHandle) itemhandle));
	}
	if (varyallseen) {
		GetDItem(variantswin, diVariantsSeeAll, &itemtype, &itemhandle, &itemrect);
		set_g_see_all(GetCtlValue((ControlHandle) itemhandle));
	}
	/* Implement the random checkbox variants. */
	for (i = 0; i < numcheckboxes; ++i) {
		GetDItem(variantswin, diVariantsFirstCheckBox + i, &itemtype, &itemhandle, &itemrect);
		if (GetCtlValue((ControlHandle) itemhandle)) {
			for (varrest = cdr(checkboxes[i]); varrest != lispnil; varrest = cdr(varrest)) {
				interp_form(NULL, car(varrest));
			}
		}
	}
	/* (should implement the random slider variants) */
	/* Recheck everything, the variants might have broken something. */
	if (anyvariants) {
		check_game_validity();
	}
}

/* This is a callback that draws the world and its areas. */

pascal void
draw_world_picture(win, ditem)
WindowPtr win;
short ditem;
{
	Point dims, center;
	Rect worldrect, arearect;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	GetDItem(worldshapewin, diWorldShapePicture, &itemtype, &itemhandle, &itemrect);
	/* Make a framed gray background for the world picture. */
	FillRect(&itemrect, QD(gray));
	FrameRect(&itemrect);
	InsetRect(&itemrect, 10, 10);
	dims.h = itemrect.right - itemrect.left;  dims.v = itemrect.bottom - itemrect.top;
	center.h = itemrect.left + dims.h / 2;  center.v = itemrect.top + dims.v / 2;
	worldrect = itemrect;
	/* Draw the whole world as a circle if it's not too big. */
	if (newcircumference < newwidth) {
		/* If world circumference smaller than area, world is not being used at all. */
	} else if (newcircumference / 2 <= dims.h) {
		InsetRect(&worldrect, (dims.h - newcircumference / 2) / 2, (dims.v - newcircumference / 2) / 2);
		EraseOval(&worldrect);
		FrameOval(&worldrect);
	} else {
		EraseRect(&worldrect);
	}
	/* Draw some grid lines. */
	MoveTo(worldrect.left, center.v);
	LineTo(worldrect.right, center.v);
	MoveTo(center.h, worldrect.top);
	LineTo(center.h, worldrect.bottom);
	/* should define an hexagon area drawer (borrow from map?). */
	arearect.top = center.v - min(newheight, newcircumference/2) / 2;
	arearect.left = center.h - min(newwidth, newcircumference/2) / 2;
	arearect.bottom = center.v + min(newheight, newcircumference/2) / 2;
	arearect.right = center.h + min(newwidth, newcircumference/2) / 2;
	arearect.top -= newlatitude;  arearect.left += newlongitude;
	arearect.bottom -= newlatitude;  arearect.right += newlongitude;
	FillRect(&arearect, QD(dkGray));
}

/* This dialog asks for the shape and size of a world to generate randomly. */

world_shape_dialog()
{
	int done = FALSE, maybechanged = TRUE, changed = FALSE;
	Str255 dlgtext;
	short ditem;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	if (worldshapewin == nil) {
		worldshapewin = GetNewDialog(dWorldShape, NULL, (DialogPtr) -1L);
		GetDItem(worldshapewin, diWorldShapePicture, &itemtype, &itemhandle, &itemrect);
		SetDItem(worldshapewin, diWorldShapePicture, itemtype, (Handle) draw_world_picture, &itemrect);
		/* Plug all the starting values into dialog items. */
		GetDItem(worldshapewin, diWorldShapeCircumference, &itemtype, &itemhandle, &itemrect);
		NumToString(newcircumference, dlgtext);
		SetIText(itemhandle, &dlgtext);
		GetDItem(worldshapewin, diWorldShapeWidth, &itemtype, &itemhandle, &itemrect);
		NumToString(newwidth, dlgtext);
		SetIText(itemhandle, &dlgtext);
		GetDItem(worldshapewin, diWorldShapeHeight, &itemtype, &itemhandle, &itemrect);
		NumToString(newheight, dlgtext);
		SetIText(itemhandle, &dlgtext);
		GetDItem(worldshapewin, diWorldShapeLatitude, &itemtype, &itemhandle, &itemrect);
		NumToString(newlatitude, dlgtext);
		SetIText(itemhandle, &dlgtext);
		GetDItem(worldshapewin, diWorldShapeLongitude, &itemtype, &itemhandle, &itemrect);
		NumToString(newlongitude, dlgtext);
		SetIText(itemhandle, &dlgtext);
	}
	while (!done) {
		/* adjust items to reflect current status */
		if (maybechanged) {
			/* Dig the values out of the dialog boxes. */
			GetDItem(worldshapewin, diWorldShapeCircumference, &itemtype, &itemhandle, &itemrect);
			GetIText(itemhandle, &dlgtext);
			StringToNum(dlgtext, &newcircumference);
			GetDItem(worldshapewin, diWorldShapeWidth, &itemtype, &itemhandle, &itemrect);
			GetIText(itemhandle, &dlgtext);
			StringToNum(dlgtext, &newwidth);
			GetDItem(worldshapewin, diWorldShapeHeight, &itemtype, &itemhandle, &itemrect);
			GetIText(itemhandle, &dlgtext);
			StringToNum(dlgtext, &newheight);
			GetDItem(worldshapewin, diWorldShapeLatitude, &itemtype, &itemhandle, &itemrect);
			GetIText(itemhandle, &dlgtext);
			StringToNum(dlgtext, &newlatitude);
			GetDItem(worldshapewin, diWorldShapeLongitude, &itemtype, &itemhandle, &itemrect);
			GetIText(itemhandle, &dlgtext);
			StringToNum(dlgtext, &newlongitude);
			DrawDialog(worldshapewin); /* just do picture item */
			maybechanged = FALSE;
		}
		/* If the proposed shape is valid, enable the OK button. */
		/* (still need to disable returns etc) */
		GetDItem(worldshapewin, diWorldShapeOK, &itemtype, &itemhandle, &itemrect);
		HiliteControl((ControlHandle) itemhandle,
					  (valid_area_shape(newwidth, newheight, FALSE) ? 0 : 255));
		/* (should filter and handle useritem events?) */
		draw_default_button(worldshapewin, diWorldShapeOK);
		ModalDialog(NULL, &ditem);
		switch (ditem) {
			case diWorldShapeOK:
				changed = TRUE;
				/* Fall through to next case. */
			case diWorldShapeCancel:
				done = TRUE;
				break;
			case diWorldShapeCircumference:
			case diWorldShapeWidth:
			case diWorldShapeHeight:
			case diWorldShapeLatitude:
			case diWorldShapeLongitude:
				maybechanged = TRUE;
				break;
			default:
				break;
		}
	}
	DisposDialog(worldshapewin);
	worldshapewin = nil;
	return changed;
}

display_world_shape_text()
{
	/* (should append to blurb) */
#if 0
	short itemtype;  Handle itemhandle;  Rect itemrect;
	Str255 tmpstr;

	GetDItem(newgamewin, diNewGameWorldSizeText, &itemtype, &itemhandle, &itemrect);
	if (varyworld) {
		sprintf(spbuf, "Diam %d, Area %dx%d @ %d,%d",
			    newcircumference, newwidth, newheight, newlongitude, newlatitude);
	} else {
		strcpy(spbuf, "");
	}
	c2p(spbuf, tmpstr);
	SetIText(itemhandle, tmpstr);
#endif
}

/* This is a callback that displays the list of sides. */

pascal void
draw_player_setup_list(win, ditem)
WindowPtr win;
short ditem;
{
	int i, strength = -1;
	Rect entryrect, tmprect;
	char *sidetitle, *playertitle;
	Player *player;
	Side *side;
	Str255 tmpstr;
	RgnHandle tmprgn;
	GrafPtr oldport;
	short itemtype;  Handle itemhandle;  Rect itemrect;

/*	GetPort(&oldport);
	SetPort(playersetupwin); */
	GetDItem(playersetupwin, diPlayerSetupList, &itemtype, &itemhandle, &itemrect);
	EraseRect(&itemrect);
	tmprgn = NewRgn();
	GetClip(tmprgn);
	ClipRect(&itemrect);
	entryrect = itemrect;
	entryrect.bottom = entryrect.top + playerh;
	for (i = 0; i < numsides; ++i) {
		side = assignments[i].side;
		draw_side_emblem(playersetupwin, entryrect.left + 2, entryrect.top + 2,
						 16, 16, side);
		sidetitle = (char *) short_side_title(side);
		if (sidetitle == NULL || strlen(sidetitle) == 0) sidetitle = "(unnamed)";
		c2p(sidetitle, tmpstr);
		MoveTo(entryrect.left + playerh, entryrect.top + playerbaseline);
		TextFont(newYork);
		TextFace(0);
		DrawString(tmpstr);
		/* (should draw some sort of icon for players) */
		player = assignments[i].player;
		playertitle = (char *) long_player_title(player);
		if (playertitle == NULL || strlen(playertitle) == 0) playertitle = "(nobody)";
		c2p(playertitle, tmpstr);
		MoveTo(entryrect.left + 2 + 120, entryrect.top + playerbaseline);
		TextFace(italic);
		DrawString(tmpstr);
		sprintf(spbuf, "%d", player->strength);
		c2p(spbuf, tmpstr);
		MoveTo(entryrect.left + 2 + 240, entryrect.top + playerbaseline);
		TextFace(0);
		DrawString(tmpstr);
		/* (should be able to draw/update selection separately) */
		if (i == selectedplayer) {
			tmprect = entryrect;
			FrameRect(&tmprect);
			InsetRect(&tmprect, 1, 1);
			FrameRect(&tmprect);
		}
		OffsetRect(&entryrect, 0, playerh);
	}
	SetClip(tmprgn);
	DisposeRgn(tmprgn);
	/* Shrink bottom of frame if numbers of sides is restricted. */
	itemrect.bottom = min(itemrect.top + g_sides_max() * 20, itemrect.bottom);
	FrameRect(&itemrect);
	/* Draw a dotted line to indicate lower limit on sides. */
	/* (better to dot frame border around additional sides instead?) */
	if (between(1, g_sides_min(), g_sides_max() - 1)) {
		PenPat(QD(gray));
		MoveTo(itemrect.left, itemrect.top + g_sides_min() * 20);
		Line(itemrect.right - itemrect.left, 0);
		PenNormal();
	}
/*	SetPort(oldport); */
}

pascal short
filter_player_setup(dialog, evt, itemhit)
DialogPtr dialog;
EventRecord *evt;
short *itemhit;
{
	GrafPtr oldport;
	Point pt;
	short ditem, sidenum;
	char ch;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	switch (evt->what) {
		case mouseDown:
			GetPort(&oldport);
			SetPort(playersetupwin);
			pt = evt->where;
			GlobalToLocal(&pt);
			ditem = FindDItem(playersetupwin, pt) + 1 /* shoot 'em... */;
			if (ditem == diPlayerSetupList) {
				GetDItem(playersetupwin, diPlayerSetupList, &itemtype, &itemhandle, &itemrect);
				sidenum = (pt.v - itemrect.top) / 20;
				select_player(between(0, sidenum, numsides-1) ? sidenum : -1);
				set_player_setup_button_states();
				DrawDialog(playersetupwin);
				SetPort(oldport);
				return TRUE;
			}
			SetPort(oldport);
			break;
		case keyDown:
			ch = evt->message & charCodeMask;
			if (ch == 3 || ch == 13) {
				if (1) {
					*itemhit = diNewGameOK;
				}
				return TRUE;
			}
			break;
	}
	return FALSE;
}

select_player(n)
int n;
{
	selectedplayer = n;
	/* (update ditem directly?) */
}

/* Bring up a dialog that allows rearrangement of the side/player assignments.
   Returns true if accepted, false if cancelled. */

player_setup_dialog()
{
	int done = FALSE, playersok = FALSE;
	short ditem;
	Player *tmpplayer;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	if (playersetupwin == nil) {
		playersetupwin = GetNewDialog(dPlayerSetup, NULL, (DialogPtr) -1L);
		GetDItem(playersetupwin, diPlayerSetupList, &itemtype, &itemhandle, &itemrect);
		SetDItem(playersetupwin, diPlayerSetupList, itemtype, (Handle) draw_player_setup_list, &itemrect);
	}
	ShowWindow(playersetupwin);
	/* Stay in here until the player accepts the setup or cancels. */
	while (!done) {
		set_player_setup_button_states();
		draw_default_button(playersetupwin, diPlayerSetupOK);
		/* Sit in the modal dialog until something happens. */
		ModalDialog((ModalFilterProcPtr) filter_player_setup, &ditem);
		switch (ditem) {
			case diPlayerSetupOK:
				playersok = TRUE;
				/* Fall through. */
			case diPlayerSetupCancel:
				done = TRUE;
				break;
			case diPlayerSetupAdd:
				add_side_and_player();
				/* Automatically select the newly-created side/player pair. */
				select_player(numsides - 1);
				init_emblem_images(selected_side());
				break;
			case diPlayerSetupRemove:
				remove_side_and_player();
				break;
			case diPlayerSetupRename:
				selected_side()->name = NULL;
				selected_side()->noun = NULL;
				/* always need to clear this cache before renaming... */
				selected_side()->pluralnoun = NULL;
				selected_side()->adjective = NULL;
				make_up_side_name(selected_side());
				init_emblem_images(selected_side());
				break;
			case diPlayerSetupStrength:
				tmpplayer = selected_player();
				tmpplayer->strength = random(5) + 1;
				break;
			case diPlayerSetupAI:
				tmpplayer = selected_player();
				/* Click between mplayer and not, for now */
				if (tmpplayer->aitypename) {
					tmpplayer->aitypename = NULL;
				} else {
					tmpplayer->aitypename = "mplayer";
				}
				break;
			case diPlayerSetupExchange:
				/* (make the following a kernel routine?) */
				tmpplayer = selected_player();
				assignments[selectedplayer].player =
					assignments[(selectedplayer + 1) % numsides].player;
				assignments[(selectedplayer + 1) % numsides].player = tmpplayer;
				/* Doesn't seem like these should be needed, but they are. */
				assignments[selectedplayer].player->side =
					assignments[selectedplayer].side;
				assignments[selectedplayer].side->player =
					assignments[selectedplayer].player;
				/* Make the next player be the selected one, then multiple clicks
				   on the button moves the selected player down through the list. */
				selectedplayer = (selectedplayer + 1) % numsides;
				assignments[selectedplayer].player->side =
					assignments[selectedplayer].side;
				assignments[selectedplayer].side->player =
					assignments[selectedplayer].player;
				break;
			default:
				break;
		}
		DrawDialog(playersetupwin); /* seems to be necessary? */
	}
	DisposDialog(playersetupwin);
	playersetupwin = nil;
	return playersok;
}

/* Set the state of the various buttons and controls. */	

set_player_setup_button_states()
{
	short itemtype;  Handle itemhandle;  Rect itemrect;

	GetDItem(playersetupwin, diPlayerSetupAdd, &itemtype, &itemhandle, &itemrect);
	HiliteControl((ControlHandle) itemhandle, ((numsides < g_sides_max()) ? 0 : 255));
	GetDItem(playersetupwin, diPlayerSetupRemove, &itemtype, &itemhandle, &itemrect);
	HiliteControl((ControlHandle) itemhandle, (FALSE /* (numsides > g_sides_min()) */ ? 0 : 255));
	GetDItem(playersetupwin, diPlayerSetupEdit, &itemtype, &itemhandle, &itemrect);
	HiliteControl((ControlHandle) itemhandle, (FALSE ? 0 : 255));
	GetDItem(playersetupwin, diPlayerSetupRename, &itemtype, &itemhandle, &itemrect);
	HiliteControl((ControlHandle) itemhandle,
				  ((selected_a_player() && !(selected_side()->nameslocked)) ? 0 : 255));
	GetDItem(playersetupwin, diPlayerSetupAI, &itemtype, &itemhandle, &itemrect);
	HiliteControl((ControlHandle) itemhandle, (selected_a_player() ? 0 : 255));
	GetDItem(playersetupwin, diPlayerSetupStrength, &itemtype, &itemhandle, &itemrect);
	HiliteControl((ControlHandle) itemhandle, (selected_a_player() ? 0 : 255));
	GetDItem(playersetupwin, diPlayerSetupRemote, &itemtype, &itemhandle, &itemrect);
	HiliteControl((ControlHandle) itemhandle, (FALSE ? 0 : 255));
	GetDItem(playersetupwin, diPlayerSetupExchange, &itemtype, &itemhandle, &itemrect);
	HiliteControl((ControlHandle) itemhandle, ((selected_a_player() && numsides > 1) ? 0 : 255));
}

/* This dialog opens a player-specified file as a game. */

open_game_dialog()
{
	Point pnt;
	/* SFTypelist typelist; */
	long typelist[4];
	SFReply reply;
	int frefnum;
	char modulename[256];
	Module *module;

	/* Gotta be somewhere... */
	SetPt(&pnt, 100, 100);
	/* Game module are text files. */
	typelist[0] = 'TEXT';
#ifdef THINK_C
	SFGetFile(pnt, "\p", NULL, 1, &typelist, NULL, &reply);
#else
/*	SFGetFile(pnt, "\p", NULL, 1, (SFTypeList) &typelist, NULL, &reply); */
#endif
	if (reply.good) {
		clear_game_modules();
		/* Build a module with the given filename. */
		module = get_game_module(NULL);
		p2c(reply.fName, modulename);
		module->filename = copy_string(modulename);
		module->hook = NULL;
		/* This module is effectively the "selected game", whether or not
		   it's loadable. */
		selectedgame = module;
		/* Load the module. */
		load_game_module(module);
		/* Change cursor back, in case it was different during loading. */
		SetCursor(&QD(arrow));
		if (!module->loaded) {
			init_warning("Could not load a module from %s", module->filename);
			return FALSE;
		}
		/* If the loaded module has bugs, we will get alerts here. */
		check_game_validity();
		/* We can now merge with the "new game" startup sequence. */
		return launch_game();
	}
	return FALSE;
}

/* This is just the collection of init steps shared by both new and open dialogs. */

launch_game()
{
	int u, t, e, tot = 0;
	Side *side;

	if (g_random_state() >= 0) init_random(g_random_state());
	/* Bring up the pre-player-setup variants. */
	if (!variants_dialog()) {
		/* Cancelling from variants dialog goes back to menu bar (?) */
		return FALSE;
	}
	/* Load any colors first, we can use them to substitute for patterns. */
	init_terrain_colors();
	/* Set up the images we'll be needing. */
	init_terrain_images();
	init_unit_images();
	/* Note that trial side/player assignment will create the default player if nec. */
	make_trial_assignments();
	init_all_emblem_images();
	/* Ask for modifications to the trial assignment. */
	if (!player_setup_dialog()) {
		/* Cancelled out of player setup, need to go around again. */
		update_new_game_list();
		return FALSE;
	}
	/* Complain if any pictures were missing.  Note that absence of resource files is
	   not necessarily an error - pictures might have been added into the app itself. */
	if (missing_images(tmpbuf)) {
		sprintf(spbuf, "%s%s",
				tmpbuf, (foundresourcefile ? "" : " & no image resource files found"));
		c2p(spbuf, tmpbuf);
		ParamText(tmpbuf, "\p", "\p", "\p");
		switch (CautionAlert(1004, nil)) {
			case 1:
				/* We're going to live with garbled imagery. */
				break;
			case 2:
				/* Ugliness is unbearable, get out now. */
				ExitToShell();
				break;
		}
	}
	/* A last-minute patch to ensure reasonable area dimensions. */
	if (area.width == 0) area.width = 60;  if (area.height == 0) area.height = 30;
	open_progress_dialog();
	/* Synthesize the remainder of the initial game setup. */
	run_synth_methods();
	final_init();
	assign_players_to_sides();
	/* No more interminable init calcs (we hope), so close the progress bar. */
	close_progress_dialog();
	/* Init the display now. */
	init_display();
	ingame = TRUE;
	/* Do a kickoff run step before falling into the main event loop. */
	run_game(0);
	return TRUE;
}

/* Rotate around "marching parentheses" cursors to indicate reading is happening. */

announce_read_progress()
{
	if (readprogressors[curcurs] != nil) SetCursor(*(readprogressors[curcurs]));
	curcurs = (curcurs + 1) % NUMcParens;
}

/* Display progress in lengthy initialization processes. */

/* This is the user item that actually draws the progress bar. */

pascal void
draw_progress(win, ditem)
WindowPtr win;
short ditem;
{
	short itemtype;  Handle itemhandle;  Rect itemrect;

	GetDItem(progresswin, diProgressBar, &itemtype, &itemhandle, &itemrect);
	if (progress >= 0) {
		/* Only draw the gray bg initially or when "progress" decreases (would be a
		   bug in synth method, but useful to get visual evidence if it happens). */
		if (lastprogress < 0 || lastprogress > progress) {
			FillRect(&itemrect, QD(gray));
			FrameRect(&itemrect);
		}
		itemrect.right =
			itemrect.left + (progress * (itemrect.right - itemrect.left)) / 100;
		FillRect(&itemrect, QD(black));
	} else {
		EraseRect(&itemrect);
	}
}

/* This starts off the lengthy process. */

open_progress_dialog()
{
	short itemtype;  Handle itemhandle;  Rect itemrect;

	progresswin = GetNewDialog(dProgress, NULL, (DialogPtr) -1L);
	GetDItem(progresswin, diProgressBar, &itemtype, &itemhandle, &itemrect);
	SetDItem(progresswin, diProgressBar, itemtype, (Handle) draw_progress, &itemrect);
	/* Always disable the Stop button for now. */
	/* (should figure out what to do with this) */
	GetDItem(progresswin, diProgressCancel, &itemtype, &itemhandle, &itemrect);
	HiliteControl((ControlHandle) itemhandle, 255);
	curcurs = 0;
	if (progressors[curcurs] != nil) SetCursor(*(progressors[curcurs]));
}

/* This is called to announce the beginning of a lengthy process. */

announce_lengthy_process(msg)
char *msg;
{
	Str255 tmpstr;
	short itemtype;  Handle itemhandle;  Rect itemrect;

	if (progresswin == nil) return;  /* be safe */
	/* Paste in the description of what is going to take so long. */
	GetDItem(progresswin, diProgressText, &itemtype, &itemhandle, &itemrect);
	c2p(msg, tmpstr);
	SetIText(itemhandle, tmpstr);
	/* Initialize the progress bar itself. */
	progress = -1;
	lastprogress = 0;
	ShowWindow(progresswin);
	SelectWindow(progresswin);
	Dprintf("%s;", msg);
#ifdef USE_CONSOLE
	if (Debug) fflush(stdout);  /* or say Dfflush() ? */
#endif
	curcurs = 0;
	if (progressors[curcurs] != nil) SetCursor(*(progressors[curcurs]));
}

/* This is called periodically to say how much progress has been made. */

announce_progress(pctdone)
int pctdone;
{
	if (progresswin == nil) return;  /* be safe */
	SelectWindow(progresswin);
	lastprogress = progress;
	progress = min(max(0, pctdone), 100);
	/* Redraw the progress bar user item. */
	DrawDialog(progresswin);
	if (progressors[curcurs] != nil) SetCursor(*(progressors[curcurs]));
	curcurs = (curcurs + 1) % NUMcSynth;
	Dprintf(" %d%%", pctdone);
#ifdef USE_CONSOLE
	if (Debug) fflush(stdout);
#endif
}

/* This is called to signal the end of the lengthy process. */

finish_lengthy_process()
{
	short itemtype;  Handle itemhandle;  Rect itemrect;

	if (progresswin == nil) return;  /* be safe */
	SelectWindow(progresswin);
	/* Replace the text with a "reassuring" but uninformative message. */
	GetDItem(progresswin, diProgressText, &itemtype, &itemhandle, &itemrect);
	SetIText(itemhandle, "\pKeeping busy...");
	progress = -1;
	/* Force a redraw, which will cause the progress bar to be erased. */
	DrawDialog(progresswin);
	Dprintf(" done.\n");
	curcurs = 0;
	if (progressors[curcurs] != nil) SetCursor(*(progressors[curcurs]));
}

/* When we're completely done with all lengthy init processes, trash the dialog. */

close_progress_dialog()
{
	/* Restore the normal cursor. */
	SetCursor(&QD(arrow));
	DisposDialog(progresswin);
	progresswin = nil;
}

/* Kernel callback to put in a default player, assumed to be on
   "this" screen (which we call "mac", for lack of inspiration). */

Player *
add_default_player()
{
	Player *player = add_player();
	
	player->displayname = "mac";
	return player;
}

/* This is a kernel callback that sets the given side to have a user
   interface.  On the Mac, we just record the side as "dside", will
   finish opening later. */

init_ui(side)
Side *side;
{
	if (side != NULL) {
		side->ui = (UI *) xmalloc(sizeof(UI));
		side->ui->active = FALSE;
		dside = side;
	}
}

/* Flag the screen as officially "open", init random globals, and open the
   initial set of windows. */

init_display()
{
	int p;

	/* If dside never got assigned, try some hasty repairs. */
	if (dside == NULL) {
		init_warning("Nobody wanted a display!");
		/* Make an arbitrary choice. */
		init_ui(side_n(1));
		if (dside == NULL) {
			/* Geez, nothing is going right. */
			init_error("Can't set up the display!");
		}
	}
	/* Init polygon-caching machinery. */
	for (p = 0; p < NUMPOWERS; ++p) {
		polygons[p] = gridpolygons[p] = nil;
		lastpolyx[p] = lastpolyy[p] = 0;
		lastgridpolyx[p] = lastgridpolyy[p] = 0;
	}
	/* Init assorted list heads and window pointers. */
	maplist = NULL;
	listlist = NULL;
	unitcloseuplist = NULL;
	sidecloseuplist = NULL;
	historywin = nil;
	/* Flag the interface as ready to display stuff.  After this point,
	   any events causing display activity will actually be rendered
	   on the screen. */
	dside->ui->active = TRUE;
	if (0 /* restore preferred windows */) {
	} else {
		/* This needs to come up so that players unfamiliar with the game
		   will have some guidance. */
		instructions_dialog();
		/* This will be the main default display, so it should be on top. */
		create_map(5);
	}
#ifdef DESIGNERS
	/* If we're starting out designing, bring up the palette automatically. */
	if (dside->designer) {
		create_design_window();
	}
#endif /* DESIGNERS */
	Dprintf("Opened Mac display!\n");
}

init_terrain_images()
{
	int i, t;
	Handle pathandle;
	PicHandle pichandle;
	Image *img;

	timages = (ImageFamily *) xmalloc(numttypes * sizeof(ImageFamily));

	for_all_terrain_types(t) {
		init_image_family(&(timages[t]));
		/* (should be in kernel) */
		if (t_image_name(t) != NULL && strlen(t_image_name(t)) > 0) {
			timages[t].name = t_image_name(t);
/*		} else if (t_color_defined(t)) {
			timages[t].name = "none";  */
		} else {
			timages[t].name = t_type_name(t);
		}
		load_image_family(&(timages[t]));
		/* Make a default pattern of box with ttype number in binary inside. */
		if (timages[t].numsizes == 0) {
			/* Count this type as not having a pattern, but only if we didn't
				find a color for it either. */
			if (!t_color_defined(t)) record_missing_image(TTYP, t_type_name(t));
			/* Build a substitute pattern. */
			img = &(timages[t].images[(timages[t].numsizes)++]);
			img->w = img->h = 8;
			img->minw = img->minh = 1;
			img->maxw = img->maxh = 9999;
			img->pat[0] = 0x00;
			img->pat[1] = 0x7f;
			img->pat[2] = 0x41;
			img->pat[3] = 0x40 | (t << 1) | 0x01;
			img->pat[4] = 0x40 | (t << 1) | 0x01;
			img->pat[5] = 0x40 | (t << 1) | 0x01;
			img->pat[6] = 0x41;
			img->pat[7] = 0x7f;
			img->patdefined = TRUE;
		}
	}
}

/* Allocate space and load a named color for each terrain type
   that defines one. */

init_terrain_colors()
{
	int t;

	if (hasColorQD) {
		tcolors = (ImageColor **) xmalloc(numttypes * sizeof(ImageColor *));
		for_all_terrain_types(t) {
			tcolors[t] = NULL;
			if (t_color(t) != NULL && strlen(t_color(t)) > 0) {
				tcolors[t] = (ImageColor *) xmalloc(sizeof(ImageColor));
				tcolors[t]->name = t_color(t);
				load_image_color(tcolors[t]);
			}
		}
	}
}

/* Allocate and fill in images for all the unit types. */

init_unit_images()
{
	int u, i;
	Rect tmprect;
	Str255 tmpstr;
	PicHandle pichandle;
	Image *img;
	Handle sicnhandle, maskhandle;

	uimages = (ImageFamily *) xmalloc(numutypes * sizeof(ImageFamily));

	for_all_unit_types(u) {
		init_image_family(&(uimages[u]));
		/* (should be in kernel) */
		if (u_image_name(u) && strlen(u_image_name(u)) > 0) {
			uimages[u].name = u_image_name(u);
		} else {
			uimages[u].name = u_internal_name(u);
		}
		load_image_family(&(uimages[u]));
		/* Default unit image binary-encodes the type number inside a box shape. */
		if (uimages[u].numsizes == 0) {
			record_missing_image(UTYP, u_type_name(u));
			img = &(uimages[u].images[(uimages[u].numsizes)++]);
#if 0
			img->w = img->h = 16;
			img->minw = img->minh = 8;
			img->maxw = img->maxh = 64;
			sicnhandle = NewHandle(32);
			(*sicnhandle)[0] = (*sicnhandle)[1] = 0;
			(*sicnhandle)[2] = 0x7f;  (*sicnhandle)[3] = 0xfe;
			for (i = 4; i < 28; ++i) {
				(*sicnhandle)[i] = (i % 2 ? 0x02 : 0x40);
			}
			(*sicnhandle)[28] = 0x7f;  (*sicnhandle)[29] = 0xfe;
			(*sicnhandle)[30] = (*sicnhandle)[31] = 0;
			/* Insert a underlined binary code for the utype. */
			(*sicnhandle)[16] |= u;
			(*sicnhandle)[18] |= u;
			(*sicnhandle)[20] |= u;
			(*sicnhandle)[22] |= u;
			(*sicnhandle)[24] |= 0x3f;
			img->monosicn = sicnhandle;
			/* Also make an 14x14 square that will be the mask. */
			maskhandle = NewHandle(32);
			(*maskhandle)[0] = (*maskhandle)[1] = 0;
			for (i = 2; i < 30; ++i) {
				(*maskhandle)[i] = (i % 2 ? 0xfe : 0x7f);
			}
			(*maskhandle)[30] = (*maskhandle)[31] = 0;
			img->masksicn = maskhandle;
#endif
		{
			RgnHandle tmprgn = NewRgn();
			GetClip(tmprgn);
			SetRect(&tmprect, 0, 0, 64, 64);
			ClipRect(&tmprect);
			pichandle = OpenPicture(&tmprect);
			FillRect(&tmprect, QD(white));
			InsetRect(&tmprect, 2, 2);
			FrameRect(&tmprect);
			MoveTo(4, 32);
			c2p(u_type_name(u), tmpstr);
			DrawString(tmpstr);
			ClosePicture();
			SetClip(tmprgn);
			DisposeRgn(tmprgn);
		}
			img->monopict = pichandle;
			img->istile = FALSE;
		}
	}
}

/* Load/create all the images of each side's emblem. */

init_all_emblem_images()
{
	Side *side2;

	eimages = (ImageFamily *) xmalloc((MAXSIDES + 1) * sizeof(ImageFamily));

	for_all_sides_plus_indep(side2) {
		init_emblem_images(side2);
	}
}

init_emblem_images(side2)
Side *side2;
{
	int e, i;
	Image *img;
	Handle sicnhandle, maskhandle;
	Str255 tmpstr;
	
	e = side_number(side2);
	init_image_family(&(eimages[e]));
	/* (should be in kernel) */
	if (side2 != NULL) {
		if (side2->emblemname && strlen(side2->emblemname) > 0) {
			eimages[e].name = side2->emblemname;
		}
	} else {
		if (indepside->emblemname && strlen(indepside->emblemname) > 0) {
			eimages[e].name = indepside->emblemname;
		}
	}
	load_image_family(&(eimages[e]));
	/* Default image is SICN-sized number.  Use only if no images have been found
	   and the emblem name is not "none". */
	if (eimages[e].numsizes == 0
		&& !(eimages[e].name && strcmp(eimages[e].name, "none") == 0)) {
		img = &(eimages[e].images[(eimages[e].numsizes)++]);
		img->w = img->h = 16;
		img->minw = img->minh = 8;
		img->maxw = img->maxh = 64;
		sprintf(tmpbuf, "s%d", e);
		c2p(tmpbuf, tmpstr);
		if ((sicnhandle = GetNamedResource('SICN', tmpstr)) != nil) {
		} else {
			record_missing_image(3, tmpbuf); /* not quite in the right place? */
			/* If we can't even get a number, make junk. */
			sicnhandle = NewHandle(32);
			for (i = 0; i < 32; i += 2) {
				(*sicnhandle)[i] = e;
			}
			(*sicnhandle)[0] = (*sicnhandle)[1] = 0xff;
			(*sicnhandle)[2] = (*sicnhandle)[3] = 0xff;
			(*sicnhandle)[12] = (*sicnhandle)[13] = 0xff;
			(*sicnhandle)[14] = (*sicnhandle)[15] = 0xff;
		}
		img->monosicn = sicnhandle;
		/* (should also be able to find number's mask? second half of sicn?) */
		maskhandle = NewHandle(32);
		for (i = 0; i < 32; ++i) {
			(*maskhandle)[i] = 0xff;
		}
		img->masksicn = maskhandle;
		img->istile = FALSE;
	}
}

/* (should go to kernel, init.c probably) */

#define NUMTOLIST 5

char *missinglist = NULL;

int missing[4];

int totlisted = 0;

record_missing_image(typtyp, str)
int typtyp;
char *str;
{
	if (missinglist == NULL) {
		missinglist = (char *) xmalloc(BUFSIZE + sizeof(char));
		missinglist[0] = '\0';
	}
	++missing[typtyp];
	/* Add the name of the image-less type, but only if one of the first few. */
	if (between(1, totlisted, NUMTOLIST)) strcat(missinglist, ",");
	if (totlisted < NUMTOLIST) {
		strcat(missinglist, str);
	} else if (totlisted == NUMTOLIST) {
		strcat(missinglist, "...");
	}
	++totlisted;
}

/* Return true if any images could not be found, and provide some helpful info
   into the supplied buffer. */

missing_images(buf)
char *buf;
{
	if (missinglist == NULL) return FALSE;
	sprintf(buf, "");
	/* (should make into a loop?) */
	if (missing[UTYP] > 0)
	  sprintf(buf+strlen(buf), " %d unit images", missing[UTYP]);
	if (missing[TTYP] > 0)
	  sprintf(buf+strlen(buf), " %d terrain images", missing[TTYP]);
	if (missing[3] > 0)
	  sprintf(buf+strlen(buf), " %d emblems", missing[3]);
	sprintf(buf+strlen(buf), " - %s", missinglist);
	return TRUE;
}
