/* Copyright (c) 1992  Stanley T. Shebs. */
/* This program may be used, copied, modified, and redistributed freely */
/* for noncommercial purposes, so long as this notice remains intact. */

/* Map graphics for the Mac interface. */

#include "conq.h"
#include "mac.h"

/* (should be adjusted for expected range of values sometimes) */

int curactcolw = 10;
int imagecolw = 20;
int namecolw = 100;
int typecolw = 70;
int sidecolw = 70;
int hpcolw = 40;
int acpcolw = 60;
int poscolw = 60;
int supcolw = 40;

int listtoph = 20;
int entryspacing;
int smallentryspacing = 18;
int largeentryspacing = 34;

int listnum = 1;

create_list()
{
	int i, numvisunits;
	List *list = (List *) xmalloc(sizeof(List));
	Rect hscrollrect, vscrollrect;

	DGprintf("Creating a list\n");
	list->sides == -1L;  /* lists every side and indeps too */
	for (i = 0; i < 5; ++i) {
		list->sortkeys[i] = bynothing;
	}
	list->listglimpsed = FALSE;
	init_list_contents(list);
	organize_list_contents(list);
	list->next = listlist;
	listlist = list;
	/* Make a window for the list, give it scrollbars. */
	list->window = GetNewWindow(wList, nil, (WindowPtr) -1L);
	ShowWindow(list->window);
	SetPort(list->window);
	hscrollrect = list->window->portRect;
	hscrollrect.top = hscrollrect.bottom - sbarwid;
	hscrollrect.bottom += 1;
	hscrollrect.left += (curactcolw + imagecolw + namecolw) + 1;
	hscrollrect.right -= sbarwid - 1;
	list->hscrollbar = NewControl(list->window, &hscrollrect, "\p", TRUE,
			 					  0, 0, 300, scrollBarProc, 0L);
	vscrollrect = list->window->portRect;
	vscrollrect.top -= 1;
	vscrollrect.bottom -= sbarwid - 1;
	vscrollrect.left = vscrollrect.right - sbarwid;
	vscrollrect.right += 1;
	numvisunits = (vscrollrect.bottom - vscrollrect.top) / smallentryspacing;
	list->vscrollbar = NewControl(list->window, &vscrollrect, "\p", TRUE,
			 					  0, 0, max(0, list->numunits - numvisunits), scrollBarProc, 0L);
	set_list_scrollbar(list);
	list->firstvisible = 0;
	sprintf(spbuf, "List %d", listnum++);
	add_window_menu_item(spbuf, list->window);
}

/* Make the list be empty. */

init_list_contents(list)
List *list;
{
	int i;

	for (i = 0; i < MAXINLIST; ++i) {
		list->units[i] = NULL;
		list->selected[i] = FALSE;
	}
	list->numunits = 0;
}

List *tmplist;

/* (should go through sort keys in selected order) */
/* (this should maybe be a kernel routine compare_units_by_keys,
   set of keys tmpkeys[] is universal) */

compare_list_entries(e1, e2)
Unit **e1, **e2;
{
	Unit *unit1 = *e1, *unit2 = *e2;

	if (unit1 == unit2) return 0;
	if (unit1 == NULL) return 1;
	if (unit2 == NULL) return -1;
	if (unit1->side != unit2->side) {
		int s1 = side_number(unit1->side), s2 = side_number(unit2->side);
		
		/* Put independents at the end of any list. */
		if (s1 == 0) s1 = numsides + 1;
		if (s2 == 0) s2 = numsides + 1;
		return (s1 - s2);
	}
	if (unit1->type != unit2->type)
		return (unit1->type - unit2->type);
	if (unit1->name) {
		if (unit2->name) {
			return strcmp(unit1->name, unit2->name);
		} else {
			return -1;
		}
	} else if (unit1->number > 0) {
		if (unit2->name) {
			return 1;
		} else if (unit2->number > 0) {
			return (unit1->number - unit2->number);
		} else {
			return -1;
		}
	} else if (unit2->name) {
		return 1;
	} else if (unit2->number > 0) {
		return 1;
	} else {
		/* Unit ids are all unique, so this is a reliable backup sort key. */
		return (unit1->id - unit2->id);
	}
	return 0;
}

/* This takes the list and fills in the items it is to display. */

organize_list_contents(list)
List *list;
{
	Side *side2;
	Unit *unit;

	/* Build up the array of units for this list. */
	list->numunits = 0;
	/* We always see our own units. */
	for_all_side_units(dside, unit) {
		add_unit_to_list(list, unit);
	}
	for_all_sides(side2) {
		if (dside != side2) {
			for_all_side_units(side2, unit) {
				if (side_sees_image(dside, unit)) {
					add_unit_to_list(list, unit);
				}
			}
		}
	}
	for_all_side_units(indepside, unit) {
		if (side_sees_image(dside, unit)) {
			add_unit_to_list(list, unit);
		}
	}
	/* Now sort the list according to its keys. */
	tmplist = list;
	qsort(list->units, list->numunits, sizeof(Unit *), compare_list_entries);
}

add_unit_to_list(list, unit)
List *list;
Unit *unit;
{
	if (list->numunits < MAXINLIST) {
		if (alive(unit)) {
			/* (should apply other inclusion criteria too) */
			list->selected[list->numunits] = FALSE;
			list->units[list->numunits++] = unit;
		}
	}
}

set_list_scrollbar(list)
List *list;
{
	/* (adjust scrollbars to fit changed list size) */
}

List *
list_from_window(window)
WindowPtr window;
{
	List *list;
	
	if (dside == NULL) return NULL;
	for_all_lists(list) {
		if (list->window == window) return list;
	}
	return NULL;
}

draw_list(list)
List *list;
{
	WindowPtr listwin;
	Rect tmprect, cliprect;
	RgnHandle tmprgn;

	listwin = list->window;
	tmprgn = NewRgn();
	GetClip(tmprgn);
	tmprect = listwin->portRect;
	tmprect.right -= sbarwid;
	BackPat(QD(white));
	EraseRect(&tmprect);
	/* Set up clipping for the contents of the list. */
	cliprect = listwin->portRect;
	cliprect.right -= sbarwid;
	ClipRect(&cliprect);
	TextSize(9);
	draw_list_contents(list);
	SetClip(tmprgn);
	DisposeRgn(tmprgn);
}
	
draw_list_contents(list)
List *list;
{
	int line, numvisunits;
	int x = 0;
	Rect tmprect;

	tmprect = list->window->portRect;
	EraseRect(&tmprect);
	entryspacing = (list->largeicons ? largeentryspacing : smallentryspacing);
	/* Image is basically square, but add a bit of extra space on each side. */
	imagecolw = entryspacing + 2;
	/* Draw the selection and sorting as a sort of header. */
	draw_list_headings(list);
	/* Compute how many list elements are actually visible. */
	numvisunits = (tmprect.bottom - tmprect.top - listtoph - sbarwid) / entryspacing;
	numvisunits = min(numvisunits, list->numunits);
	list->lastvisible = list->firstvisible + numvisunits - 1;
	for (line = list->firstvisible; line <= list->lastvisible; ++line) {
		draw_unit_list_entry(list, line, FALSE);
	}
	HiliteControl(list->vscrollbar, ((numvisunits < list->numunits) ? 0 : 255));
}

draw_list_headings(list)
List *list;
{
	int x = 0, m;
	Str255 tmpstr;

	/* (should underline sort keys with varying line heaviness) */
	/* We have to do MoveTo everywhere because DrawString moves the pen. */
	x += curactcolw;
	MoveTo(x, 15);
	DrawString("\ps/L");
	x += imagecolw;
	MoveTo(x, 15);
	DrawString("\pName/Number");
	x += namecolw;
	MoveTo(x, 15);
	DrawString("\pType");
	x += typecolw;
	MoveTo(x, 15);
	DrawString("\pSide");
	x += sidecolw;
	MoveTo(x, 15);
	DrawString("\pHp");
	x += hpcolw;
	MoveTo(x, 15);
	DrawString("\pAcp");
	x += acpcolw;
	MoveTo(x, 15);
	DrawString("\pLoc");
	x += poscolw;
	for_all_material_types(m) {
		MoveTo(x, 15);
		c2p(m_type_name(m), tmpstr);
		DrawString(tmpstr);
		x += supcolw;
	}
	/* Draw a dividing line. */
	MoveTo(0, listtoph);
	Line(list->window->portRect.right, 0);
}

/* This draws a one-line entry for the given unit. */

draw_unit_list_entry(list, n, clearfirst)
List *list;
int n, clearfirst;
{
	int u, m, x, y = (n - list->firstvisible) * entryspacing + listtoph;
	int texty = y + 15;
	Side *side2;
	Rect entryrect, tmprect;
	Unit *unit = list->units[n];

	SetRect(&entryrect, 0, y, list->window->portRect.right, y + entryspacing);
	if (clearfirst) {
		EraseRect(&entryrect);
	}
	if (unit == NULL || !alive(unit)) {
		/* We need to recalculate the list contents. */
		list->shouldreorg = TRUE;
		return;
	}
	u = unit->type;
	/* Draw whether the unit is awake or asleep. */
	if (unit->plan && completed(unit)) {
		SetRect(&tmprect, 0, y + entryspacing/2 - curactcolw/2,
				curactcolw, y + entryspacing/2 + curactcolw/2);
		InsetRect(&tmprect, 2, 2);
		if (unit->plan->asleep) {
		} else if (unit->plan->reserve) {
			FillRect(&tmprect, QD(gray));
		} else {
			FillRect(&tmprect, QD(black));
		}
		FrameRect(&tmprect);
	}
	/* Draw an icon with side emblem for this unit. */
	draw_unit_image(list->window, curactcolw + 2, y + 1,
					(list->largeicons ? 32 : 16), (list->largeicons ? 32 : 16),
					u, side_number(unit->side), !completed(unit));
	/* Write the name or ordinal number. */
	if (unit->name) {
		sprintf(spbuf, "%s", unit->name);
	} else if (unit->number > 0) {
		sprintf(spbuf, "%d%s", unit->number, ordinal_suffix(unit->number));
	} else {
		sprintf(spbuf, "");
	}
	spbuf[15] = '\0';  /* (should be clipping by pixels) */
	x = curactcolw + imagecolw;
	MoveTo(x, texty);
	c2p(spbuf, tmpbuf);
	DrawString(tmpbuf);
	/* Write the name of the unit's type. */
	x += namecolw;
	MoveTo(x, texty);
	sprintf(spbuf, "%s", u_type_name(u));
	spbuf[10] = '\0';
	c2p(spbuf, tmpbuf);
	DrawString(tmpbuf);
	/* Write the side of the unit. */
	x += typecolw;
	MoveTo(x, texty);
	side2 = unit->side;
	shortest_side_title(side2, spbuf);
	spbuf[15] = '\0'; /* truncate long side names */
	c2p(spbuf, tmpbuf);
	DrawString(tmpbuf);
	/* Draw the current hit points of the unit. */
	x += sidecolw;
	if (side_sees_unit(dside, unit)) {
		MoveTo(x, texty);
		/* (kernel candidate?) */
		if (u_hp(u) > 0 && unit->hp > 0) {
			if (unit->hp == u_hp(u)) {
				sprintf(spbuf, "%d", unit->hp);
			} else {
				sprintf(spbuf, "%d/%d", unit->hp, u_hp(u));
			}
		} else {
			sprintf(spbuf, "-");
		}
		c2p(spbuf, tmpbuf);
		DrawString(tmpbuf);
	}
	x += hpcolw;
	if (side_sees_unit(dside, unit)) {
		MoveTo(x, texty);
		/* (kernel candidate?) */
		if (completed(unit)) {
			if (unit->act && unit->act->initacp > 0) {
				if (unit->act->acp == unit->act->initacp) {
					sprintf(spbuf, "%d", unit->act->acp);
				} else {
					sprintf(spbuf, "%d/%d", unit->act->acp, unit->act->initacp);
				}
			} else {
				sprintf(spbuf, "-");
			}
			c2p(spbuf, tmpbuf);
			DrawString(tmpbuf);
		} else {
			sprintf(spbuf, "%d/%d done", unit->cp, u_cp(u));
			c2p(spbuf, tmpbuf);
			DrawString(tmpbuf);
			/* Still need this to move over to the next column. */
		}
	}
	/* Draw the location. */
	x += acpcolw;
	MoveTo(x, texty);
	sprintf(spbuf, "%d,%d", unit->x, unit->y);
	if (unit->z != 0) {
		sprintf(spbuf+strlen(spbuf), ",%d", unit->z);
	}
	c2p(spbuf, tmpbuf);
	DrawString(tmpbuf);
	/* Draw the state of all the supplies. */
	x += poscolw;
	for_all_material_types(m) {
		MoveTo(x, texty);
		sprintf(spbuf, "%d", unit->supply[m]);
		c2p(spbuf, tmpbuf);
		DrawString(tmpbuf);
		x += supcolw;
	}
	/* Highlight this entry if it was selected. */
	if (list->selected[n]) {
		InvertRect(&entryrect);
	}
}

grow_list(list, w, h)
List *list;
int w, h;
{
	EraseRect(&list->window->portRect);
	SizeWindow(list->window, w, h, 1);
	MoveControl(list->hscrollbar, (curactcolw+imagecolw+namecolw), h - sbarwid);
	SizeControl(list->hscrollbar, w - (curactcolw+imagecolw+namecolw) - sbarwid + 1, sbarwid + 1);
	MoveControl(list->vscrollbar, w - sbarwid, listtoph);
	SizeControl(list->vscrollbar, sbarwid + 1, h - listtoph - sbarwid + 1);
	InvalRect(&list->window->portRect);
}

/* Zooming should work like a list view in the Finder. */

zoom_window_list(list, part)
List *list;
short part;
{
	int bestw = 500, besth;

	entryspacing = (list->largeicons ? largeentryspacing : smallentryspacing);
	besth = listtoph + (list->lastvisible - list->firstvisible + 2) * entryspacing;
	if (1 /* would be off screen */) {
		ZoomWindow(list->window, part, FALSE);
	}
	grow_list(list, bestw, min(400 /* should be from screenbits */, besth));
}

List *curlist = NULL;

pascal void
list_scroll_proc(control, code)
ControlHandle control;
short code;
{
	int curvalue, minvalue, maxvalue, pagesize, dir, jump;

	if (curlist == NULL) return;

	curvalue = GetCtlValue(control);
	minvalue = GetCtlMin(control);
	maxvalue = GetCtlMax(control);
	pagesize = curlist->lastvisible - curlist->firstvisible;
	switch (code) {
		case inPageDown:
			jump = pagesize - 2;
			break;
		case inDownButton:
			jump = 1;
			break;
		case inPageUp:
			jump = - (pagesize - 2);
			break;
		case inUpButton:
			jump = -1;
			break;
		default:
			jump = 0;
			break;
	}
	curvalue = max(min(curvalue + jump, maxvalue), minvalue);
	curlist->firstvisible = curvalue;
	curlist->lastvisible = curlist->firstvisible + pagesize;
	if (curlist->lastvisible >= curlist->numunits)
		curlist->lastvisible = curlist->numunits - 1;
	SetCtlValue(control, curvalue);
	curlist = NULL;
}

/* Handle a mouse down in the list.  Grafport already set, mouse coords are local. */

/* (mouse downs should select/deselect list elements) */

do_mouse_down_list(list, mouse, mods)
List *list;
Point mouse;
int mods;
{
	ControlHandle control;
	short part, value;
	WindowPtr window = list->window;

	part = FindControl(mouse, window, &control);
	if (control == list->vscrollbar) {
		switch (part) {
			case inThumb:
				part = TrackControl(control, mouse, NULL);
				list->firstvisible = GetCtlValue(control);
				force_update(window);
				break;
			default:
				curlist = list;
				part = TrackControl(control, mouse, (ProcPtr) list_scroll_proc);
				force_update(window);
				break;
		}
	} else if (control == list->hscrollbar) {
		switch (part) {
			case inThumb:
				part = TrackControl(control, mouse, NULL);
				break;
			default:
				curlist = list;
				part = TrackControl(control, mouse, NULL /*(ProcPtr) list_scroll_proc*/);
				break;
		}
	} else {
		if (mouse.v < listtoph) {
			if (between(curactcolw, mouse.h, curactcolw + imagecolw)) {
				toggle_list_large_icons(list);
			}
			/* do others eventually */
		} else {
			int n, tmp;

			/* Figure out the selected unit. */
			n = (mouse.v - listtoph) / entryspacing + list->firstvisible;
			tmp = list->selected[n];
			clear_selections(list);
			list->selected[n] = !tmp;
			redraw_unit_list_entry(list, n);
		}
	}
}

toggle_list_large_icons(list)
List *list;
{
	list->largeicons = !list->largeicons;
	force_update(list->window);
}

update_unit_in_lists(unit)
Unit *unit;
{
	int line;
	List *list;

	for_all_lists(list) {
		if ((line = unit_position_in_list(list, unit)) >= 0) {
			if (between(list->firstvisible, line, list->lastvisible)) {
				redraw_unit_list_entry(list, line);
			}
		} else {
/*			add_unit_to_list(list, unit); */
			organize_list_contents(list);
			force_update(list->window);
		}
	}
}

unit_position_in_list(list, unit)
List *list;
Unit *unit;
{
	int i;
	
	for (i = 0; i < list->numunits; ++i) {
		if (unit == list->units[i]) return i;
	}
	return (-1);
}

redraw_unit_list_entry(list, n)
List *list;
int n;
{
	WindowPtr listwin;
	Rect cliprect;
	GrafPtr oldport;
	RgnHandle tmprgn;

	if (!active_display(dside) || list == NULL) return;

	listwin = list->window;
 	GetPort(&oldport);
	SetPort(listwin);
	tmprgn = NewRgn();
	GetClip(tmprgn);
	/* Set up clipping for the contents of the list. */
	cliprect = listwin->portRect;
	cliprect.right -= sbarwid;
	ClipRect(&cliprect);
	TextSize(9);
	draw_unit_list_entry(list, n, TRUE);
	SetClip(tmprgn);
	DisposeRgn(tmprgn);
	SetPort(oldport);
}

clear_selections(list)
List *list;
{
	int i;
	
	for (i = 0; i < list->numunits; ++i) {
		if (list->selected[i]) {
			list->selected[i] = FALSE;
			redraw_unit_list_entry(list, i);
		}
	}
}

Unit *
selected_unit_in_list(list)
List *list;
{
	int i;
	
	for (i = 0; i < list->numunits; ++i) {
		if (list->selected[i]) return list->units[i];
	}
	return NULL;
}

/* This finds a good map to scroll over to look at a unit mentioned in the list. */

scroll_to_selected_unit_in_list(list)
List *list;
{
	Unit *unit;
	Map *map, *bestmap;
	
	if ((unit = selected_unit_in_list(list)) != NULL
	    && inside_area(unit->x, unit->y)) {
		/* Find the best map to scroll over to the unit. */
		bestmap = maplist;
		for_all_maps(map) {
			if (map->power > bestmap->power
			    || (map->power == bestmap->power
			        && in_middle(map, unit->x, unit->y)
				 	&& !in_middle(bestmap, unit->x, unit->y))) {
				bestmap = map;
			}
		}
		/* We found a map, now make it show the unit. */
		if (bestmap != NULL) {
			if (!in_middle(bestmap, unit->x, unit->y)) {
				scroll_to_unit(bestmap, unit);
			}
			SelectWindow(bestmap->window);
			adjust_menus(); 
		}
	}
}
