/* Copyright (c) 1987, 1988, 1991, 1992  Stanley T. Shebs. */
/* This program may be used, copied, modified, and redistributed freely */
/* for noncommercial purposes, so long as this notice remains intact. */

/* Natural-language handling in the kernel. */
/* This file should be entirely replaced for non-English Xconqs. */

#include "conq.h"

#define g_initial_date() ((char *) g_base_date())

int baseday = -1, basemonth = 0, baseyear = 1900;

int basehour;

/* Short names of directions. */

char *dirnames[] = DIRNAMES;

/* names of columns in unit record */

char *reasonnames[] = REASONNAMES;

char sideshorttitle[BUFSIZE];

char *
short_side_title(side)
Side *side;
{
    if (side == NULL) {
	return " - ";
    } else if (side->name) {
	return side->name;
    } else if (side->pluralnoun) {
	sprintf(sideshorttitle, "The %s", side->pluralnoun);
    } else if (side->noun) {
	sprintf(sideshorttitle, "The %s", plural_form(side->noun));
    } else if (side->adjective) {
	sprintf(sideshorttitle, "The %s side", side->adjective);
    } else {
	return " - ";
    }
    return sideshorttitle;
}

char *
shortest_side_title(side2, spbuf)
Side *side2;
char *spbuf;
{
    if (side2 == NULL) {
	sprintf(spbuf, "-");
    } else if (side2->name) {
	sprintf(spbuf, "%s", side2->name);
    } else if (side2->adjective) {
	sprintf(spbuf, "%s", side2->adjective);
    } else if (side2->noun) {
	sprintf(spbuf, "%s", side2->noun);
    } else {
	sprintf(spbuf, "(#%d)", side_number(side2));
    }
    return spbuf;
}

char xxxtitlebuf[BUFSIZE];

char *
long_player_title(player)
Player *player;
{
	extern int unequalstrengths;

	if (player == NULL) {
		return "";
	} else {
		if (strcmp(player->displayname, "mac" /* fix this */) == 0) {
			sprintf(xxxtitlebuf, "You");
			if (player->aitypename) {
				sprintf(tmpbuf, "(+ %s)", player->aitypename);
				strcat(xxxtitlebuf, tmpbuf);
			}
		} else if (player->aitypename) {
			sprintf(xxxtitlebuf, "AI %s", player->aitypename);
		} else {
			sprintf(xxxtitlebuf, "-");
		}
		/* Strengths should be in a separate column. */
		if (unequalstrengths) {
			sprintf(tmpbuf, " (%d)", player->strength);
			strcat(xxxtitlebuf, tmpbuf);
		}
		return xxxtitlebuf;
	}
}

char sideandtypenamebuf[BUFSIZE];

char *
side_and_type_name(side, u, side2)
Side *side, *side2;
int u;
{
    /* Decide how to identify the side. */
    if (side2 == NULL) {
	sprintf(sideandtypenamebuf, "independent ");
    } else if (side == side2) {
	sprintf(sideandtypenamebuf, "your ");
    } else {
	sprintf(sideandtypenamebuf, "%s ",
		(side2->adjective ? side2->adjective :
		 (side2->noun ? side2->noun : "?")));
    }
    /* Glue the pieces together and return it. */
    strcat(sideandtypenamebuf, u_type_name(u));
	return sideandtypenamebuf;
}

char unitbuf[BUFSIZE];

/* Build a short phrase describing a given unit to a given side. */
/* First we supply identification of side, then of unit itself. */

char *
unit_handle(side, unit)
Side *side;
Unit *unit;
{
    char *utypename = u_type_name(unit->type);

    /* Handle various garbage conditions. */
    if (unit == NULL)
      return "???";
    if (!alive(unit) && side)
      return "?dead?";
    /* If this unit represents "yourself", say so. */
    if (side != NULL && unit == side->selfunit)
      return "you";
    /* Sometimes a unit's description should be its name alone. */
/*    if (u_name_format(unit->type) == 2 && unit->name)
      return unit->name;  */
    /* Decide how to identify the side. */
    if (unit->side == NULL) {
	sprintf(unitbuf, "the ");
    } else if (unit->side == side) {
	sprintf(unitbuf, "your ");
    } else {
	Side *side2 = unit->side;

	sprintf(unitbuf, "the %s ",
		(side2->adjective ? side2->adjective :
		 (side2->noun ? side2->noun : side_desig(side2))));
    }
    /* Now add the unit's unique description. */
    /* (Should this ever use short name?) */
    if (unit->name) {
	sprintf(tmpbuf, "%s %s", utypename, unit->name);
    } else if (unit->number > 0) {
	sprintf(tmpbuf, "%d%s %s",
		unit->number, ordinal_suffix(unit->number), utypename);
    } else {
	sprintf(tmpbuf, "%s", utypename);
    }
    /* Glue the pieces together and return it. */
    strcat(unitbuf, tmpbuf);
    return unitbuf;
}

/* Shorter unit description omits side name, but uses same buffer. */

char *
short_unit_handle(unit)
Unit *unit;
{
    int u = unit->type;

    if (unit->name && strlen(unit->name) > 0) {
	sprintf(unitbuf, "%s", unit->name);
    } else if (u_short_name(u) && strlen(u_short_name(u)) > 0) {
	sprintf(unitbuf, "%d%s %s",
		unit->number, ordinal_suffix(unit->number), u_short_name(u));
    } else {
	sprintf(unitbuf, "%d%s %s",
		unit->number, ordinal_suffix(unit->number), u_type_name(u));
    }
    return unitbuf;
}

historical_event_desc(hevt, buf)
HistEvent *hevt;
char *buf;
{
    switch (hevt->type) {
      case H_LOG_STARTED:
	sprintf(buf, "started recording events");
	break;
      case H_GAME_STARTED:
	sprintf(buf, "started the game");
	break;
      case H_SIDE_LOST:
	sprintf(buf, "%s joined", side_desig((Side *) hevt->data));
	break;
      default:
	sprintf(buf, "%s", hevtdefns[hevt->type].name);
	break;
    }
}

/* Compose a one-line comment on the game. */

char *
exit_commentary(side)
Side *side;
{
    int numingame = 0, numwon = 0, numlost = 0;
    Side *side2, *lastside, *winner, *loser;

    for_all_sides(side2) {
    if (side2->ingame) {
    	++numingame;
    	lastside = side2;
    }
	if (side_won(side2)) {
	    ++numwon;
	    winner = side2;
	}
	if (side_lost(side2)) {
	    ++numlost;
	    loser = side2;
	}
    }
    if (numingame > 0) {
    	if (0 /* could have been resolved, need to check scorekeepers */) {
			sprintf(spbuf, "The outcome remains undecided");
#ifdef RUDE
			if (numsides > 1) {
			    strcat(spbuf, ", but you're probably the loser!");
			} else {
			    strcat(spbuf, "...");
			}
#else
			strcat(spbuf, "...");
#endif /* RUDE */
		} else {
    		sprintf(spbuf, "Game is over.");
    	}
    } else if (numwon == numsides) {
	    sprintf(spbuf, "Everybody won!");
    } else if (numlost == numsides) {
	    sprintf(spbuf, "Everybody lost!");
	} else if (numwon == 1) {
		sprintf(spbuf, "%s won!", (side == winner ? "You" : side_desig(winner)));
	} else if (numlost == 1) {
		sprintf(spbuf, "%s lost!", (side == loser ? "You" : side_desig(loser)));
	} else {
		sprintf(spbuf, "Some won and some lost.");
	}
    return spbuf;
}

#if 0
#ifdef RUDE
    notify(side, "You lost, %s!", (flip_coin() ? "sucker" : "turkey"));
#else
    notify(side, "You lost!");
#endif /* RUDE */
#endif

/* The following routines should probably go into some kind of international
   type interface. */

/* Given a number, figure out what suffix should go with it. */

char *
ordinal_suffix(n)
int n;
{
    if (n % 100 == 11 || n % 100 == 12 || n % 100 == 13) {
	return "th";
    } else {
	switch (n % 10) {
	  case 1:   return "st";
	  case 2:   return "nd";
	  case 3:   return "rd";
	  default:  return "th";
	}
    }
}

/* Pluralize a word, attempting to be smart about various possibilities */
/* that don't have a different plural form (such as "Chinese" and "Swiss"). */
/* There should probably be a test for when to add "es" instead of "s". */

char *pluralbuf = NULL;

char *
plural_form(word)
char *word;
{
    char endch = ' ', nextend = ' ';

	if (pluralbuf == NULL) pluralbuf = (char *) xmalloc(BUFSIZE * sizeof(char));
    if (word == NULL) {
	run_warning("plural_form given NULL string");
	pluralbuf[0] = '\0';
	return pluralbuf;
    }
    if (strlen(word) > 0) endch   = word[strlen(word)-1];
    if (strlen(word) > 1) nextend = word[strlen(word)-2];
    if (endch == 'h' || endch == 's' || (endch == 'e' && nextend == 's')) {
	sprintf(pluralbuf, "%s", word);
    } else {
	sprintf(pluralbuf, "%ss", word);
    }
    return pluralbuf;
}


/* Date/time parsing and formatting. */

char *usual_date_string();

/* Compose a readable form of the given date. */

int calendartype = -1;

char datebuf[BUFSIZE];

char *turnname;
char *datestepname;
int datestep;

char *
absolute_date_string(date)
long date;
{
    int caltypeunknown = FALSE;
    Obj *cal, *caltype, *stepname, *step;

    /* The first time we ask for a date, interpret the calendar. */
    if (calendartype < 0) {
    	cal = g_calendar();
    	if (consp(cal)) {
    	    caltype = car(cal);
    	    if ((symbolp(caltype) || stringp(caltype))
    	    	&& strcmp("usual", c_string(caltype)) == 0) {
    	    	calendartype = 1;
    		stepname = cadr(cal);
    		datestepname = (stringp(stepname) ? c_string(stepname) : "day");
    		step = car(cddr(cal));
    		datestep = (numberp(step) ? c_number(step) : 1);
    	    } else {
    	    	caltypeunknown = TRUE;
    	    }
    	} else {
    	    calendartype = 0;
    	    turnname = "Turn";
    	    if (stringp(cal)) {
    	    	turnname = c_string(cal);
    	    } else if (cal != lispnil) {
    	    	caltypeunknown = TRUE;
    	    }
	}
	if (caltypeunknown) {
	    init_warning("Unknown calendar type");
	}
    }
    switch (calendartype) {
      case 0:
	sprintf(datebuf, "%s%4d", turnname, date);
	return datebuf;
      case 1:
	return usual_date_string(date);
      default:
	case_panic("calendar type", calendartype);
    }
}

/* Compose a date, omitting components supplied by the base date. */

char *
relative_date_string(date, base)
long date, base;
{
    /* do this for real eventually */
    sprintf(datebuf, "%d(%d)", date, base);
    return datebuf;
}

char *months[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
		   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec", "???" };

parse_usual_initial_date()
{
    char *date = g_initial_date();
    char monthname[BUFSIZE];
    int i;

    if (baseday < 0) {
      if (date != NULL && strlen(date) > 0) {
	/* Assume it's in a standard date format. */
    	if (strcmp(datestepname, "hour") == 0) {
	    sscanf(date, "%d %d %s %d", &basehour, &baseday, monthname, &baseyear);
	    --baseday;
    	} else if (strcmp(datestepname, "day") == 0) {
	    sscanf(date, "%d %s %d", &baseday, monthname, &baseyear);
	    --baseday;
    	} else if (strcmp(datestepname, "week") == 0) {
 	    sscanf(date, "%d %s %d", &baseday, monthname, &baseyear);
	    --baseday;
   	} else if (strcmp(datestepname, "month") == 0) {
 	    sscanf(date, "%s %d", monthname, &baseyear);
   	} else if (strcmp(datestepname, "year") == 0) {
	    sscanf(date, "%d", &baseyear);
    	}
	for (i = 0; i < 12; ++i) {
	    if (strcmp(monthname, months[i]) == 0) {
		basemonth = i;
		return;
	    }
	}
      } else {
	baseday = 0;
      }
    }
}


/* Given a numeric data, convert it into something understandable.  Depends
   on the length of a turn. */

char *
usual_date_string(date)
int date;
{
    int year, month = 0, week = 0, day = 0, hour = -1;

    --date;  /* first displayed date is turn 1, this adjusts */
    parse_usual_initial_date();
    /* (Short intervals should be done clock-style.) */
    if (strcmp(datestepname, "second") == 0) {
	sprintf(datebuf, "Second %d", date);
	return datebuf;
    } else if (strcmp(datestepname, "minute") == 0) {
	sprintf(datebuf, "Minute %d", date);
	return datebuf;
    } else if (strcmp(datestepname, "hour") == 0) {
 	hour = date % 24 + basehour;
 	date = date / 24;
	day = date % 365;
	day = day % 30 + 1 + baseday;
	month = day / 30 + basemonth;
	year = date / 365 + baseyear;
	sprintf(datebuf, "%d:00 %2d %s %d", hour, day, months[month], abs(year));
   } else if (strcmp(datestepname, "day") == 0) {
	/* Should do this more accurately... */
	day = date % 365;
	day = day % 30 + 1 + baseday;
	month = day / 30 + basemonth;
	year = date / 365 + baseyear;
	sprintf(datebuf, "%2d %s %d", day, months[month], abs(year));
    } else if (strcmp(datestepname, "week") == 0) {
	day = (date * 7) % 365;
	month = day / 30;
	day = day % 30 + 1;
	year = baseyear + (date * 7) / 365;
	sprintf(datebuf, "%2d %s %d", day, months[month], abs(year));
    } else if (strcmp(datestepname, "month") == 0) {
	year = baseyear + date / 12;
	sprintf(datebuf, "%s %d", months[date % 12], abs(year));
    } else if (strcmp(datestepname, "year") == 0) {
	year = baseyear + date;
	sprintf(datebuf, "%d", abs(year));
    } else {
	sprintf(datebuf, "?what's a %s?", datestepname);
    }
    if (year < 0) {
	strcat(datebuf, " BC");
    }
    return datebuf;
}

/* Return a string naming the season.  The names are wired in; for extra
   flavor, this should be settable. */

char *
season_name(turn)
int turn;
{
    int year = world.yearlength;
    int num;

    num = ((turn - 1) + (year - 0 /* g_first_midwinter() */)) % year;

    if (num <= year/2) {
	return ((num <= (1 * year) / 4) ? "winter" : "spring");
    } else {
	return ((num <= (3 * year) / 4) ? "summer" : "autumn");
    }
}

/* Display what is essentially a double-column bookkeeping of unit gains */
/* and losses.  Tricks here include the use of "dummy reason" flags to */
/* display sums of several columns. */

print_unit_record(fp, side)
FILE *fp;
Side *side;
{
#if 0
    int atype, reason, sum;

    fprintf(fp, "Unit Record (gains and losses by cause and unit type)\n");
    fprintf(fp, "   ");
    for (reason = 0; reason < NUMREASONS; ++reason) {
	fprintf(fp, " %3s", reasonnames[reason]);
    }
    fprintf(fp, "  Total\n");
    for_all_unit_types(atype) {
	sum = 0;
	fprintf(fp, " %s ", utype_name_n(atype, 1));
	for (reason = 0; reason < NUMREASONS; ++reason) {
	    if (side_balance(side, atype, reason) > 0) {
		fprintf(fp, " %3d", side_balance(side, atype, reason));
		sum += side_balance(side, atype, reason);
	    } else if (reason == DUMMYREAS) {
		fprintf(fp, " %3d", sum);
		sum = 0;
	    } else {
		fprintf(fp, "    ");
	    }
	}
	fprintf(fp, "   %3d\n", sum);
    }
    fprintf(fp, "\n");
#endif
}

/* Show some overall numbers on performance of a side. */

print_side_results(fp, side)
FILE *fp;
Side *side;
{
}

