/* This program converts the IMF interchange format to X bit/pixmaps. */

/* It requires one or more image files and a target directory for the
   bitmap files. */

#include "../src/config.h"
#include "../src/misc.h"
#include "../src/lisp.h"

#include <X11/Xos.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>

#include "ximf.h"

#define hextoi(c) (((c) >= '0' && (c) <= '9') ? ((c) - '0') : ((c) - 'a' + 10))

char *outdirname = NULL;

char imdirname[1000];

int numutypes = 0;
int nummtypes = 0;
int numttypes = 0;

int area; /* dubious... */

#define MAXIMAGES 1000

ImageFamily *images[MAXIMAGES];

int numimages = 0;

ImageFamily *get_imf();
ImageFamily *find_imf();

Image *get_img();

main(argc, argv)
int argc;
char *argv[];
{
    int i;
    FILE *ifp, *ofp;
    Obj *form;
    int startlineno = 0, endlineno = 0;

    init_lisp();

    for (i = 1; i < argc; ++i) {
	if (strcmp(argv[i], "-o") == 0) {
	    if (i + 1 < argc) {
		outdirname = argv[i + 1];
		/* Blast the arg because we'll be scanning the args again. */
		argv[i] = NULL;
		argv[i + 1] = NULL;
		++i;
	    } else {
		/* no directory name given with the -o */
	    }
	}
    }
    /* We require a -o spec, as a safety precaution. */
    if (outdirname == NULL) {
	fprintf(stderr, "Error: No output directory specified.\n");
	exit(1);
    }
    /* If the output directory does not exist, should create it? */
    for (i = 1; i < argc; ++i) {
	if (argv[i] != NULL) {
	    /* Interpret the arg as an imf file, open and read it. */
	    if ((ifp = fopen(argv[i], "r")) != NULL) {
		while ((form = read_form(ifp, &startlineno, &endlineno))
		       != lispeof) {
		    interp_imf_form(form);
		}
		fclose(ifp);
	    } else {
		run_warning("Couldn't open \"%s\", ignoring.", argv[i]);
	    }
	}
    }
    /* Write the image directory file. */
    sprintf(imdirname, "%s/%s", outdirname, "imf.dir");
    if ((ofp = fopen(imdirname, "w")) != NULL) {
	for (i = 0; i < numimages; ++i) {
	    fprintf(ofp, "%s\n", images[i]->name);
	}
	fclose(ofp);
    } else {
	/* error */
    }
}

interp_imf_form(form)
Obj *form;
{
    ImageFamily *imf;
/*    ImageColor *imc;  */
    Obj *rest, *clause, *size, *typ;
    
    if (equal(car(form), intern_symbol("imf"))) {
	if (stringp(cadr(form))) {
	    imf = get_imf(c_string(cadr(form)));
	    for (rest = cddr(form); rest != lispnil; rest = cdr(rest)) {
		clause = car(rest);
		if (consp(clause)) {
		    interp_image(imf, car(clause), cdr(clause));
		}
	    }
	} else {
	    /* garbage? */
	}
    } else if (equal(car(form), intern_symbol("palette"))) {
    } else if (equal(car(form), intern_symbol("color"))) {
	if (stringp(cadr(form))) {
#if 0
	    imc = get_imc(c_string(cadr(form)));
	    interp_color(imc, cddr(form));
#endif
	}
    } else {
	/* warn */
    }
}

interp_image(imf, size, parts)
ImageFamily *imf;
Obj *size, *parts;
{
    int w, h, i;
    char *data;
    Image *img = NULL;
    Obj *rest, *typ, *datalist;

    w = c_number(car(size));  h = c_number(cadr(size));
    if ((img = get_img(imf, w, h)) != NULL) {
	if (equal(car(cddr(size)), intern_symbol("tile"))) img->istile = TRUE;
	for (rest = parts; rest != lispnil; rest = cdr(rest)) {
	    typ = car(car(rest));
	    datalist = cdr(car(rest));
	    if (w == 8 && h == 8) {
		if (equal(typ, intern_symbol("mono"))) {
		    write_bitmap(imf->name, "8.b", 8, 8, datalist);
		}
	    } else if (w == 16 && h == 16) {
		if (equal(typ, intern_symbol("mono"))) {
		    write_bitmap(imf->name, "b", 16, 16, datalist);
		} else if (equal(typ, intern_symbol("mask"))) {
		    write_bitmap(imf->name, "m", 16, 16, datalist);
		}
	    } else if (w == 32 && h == 32) {
		if (equal(typ, intern_symbol("mono"))) {
		    write_bitmap(imf->name, "32.b", 32, 32, datalist);
		} else if (equal(typ, intern_symbol("mask"))) {
		    write_bitmap(imf->name, "32.m", 32, 32, datalist);
		}
	    }
	}
    }
}

#if 0
interp_color(imc, elts)
ImageColor *imc;
Obj *elts;
{
    imc->r = c_number(car(elts));
    imc->g = c_number(car(cdr(elts)));
    imc->b = c_number(car(cddr(elts)));
}
#endif

/* Given a name, find or create an image family with that name. */

ImageFamily *
get_imf(name)
char *name;
{
    int i;
    ImageFamily *imf = NULL;

    if (name == NULL) return NULL;
    /* Resource names ending in " mask" are parts of other images. */
    if (strcmp(name+strlen(name)-5, " mask") == 0) return NULL;
    if ((imf = find_imf(name)) == NULL) {
	if (numimages >= MAXIMAGES) {
	    /* (should realloc instead!) */
	    return;
	}
	if ((imf = (ImageFamily *) xmalloc(sizeof(ImageFamily))) != NULL) {
	    init_image_family(imf);
	    imf->name = name;
/*	    load_image_family(imf);  */
	    images[numimages++] = imf;
	}
    }
    return imf;
}

/* Find the image family of the given name, if it exists. */

ImageFamily *
find_imf(name)
char *name;
{
    int i;

    for (i = 0; i < numimages; ++i) {
	if (strcmp(name, images[i]->name) == 0) return images[i];
    }
    return NULL;
}

init_image_family(imf)
ImageFamily *imf;
{
    bzero(imf, sizeof(ImageFamily));
}

Image *
get_img(imf, w, h)
ImageFamily *imf;
int w, h;
{
    int i;
    Image *img;

    for (i = 0; i < imf->numsizes; ++i) {
	img = &(imf->images[i]);
	if (w == img->w && h == img->h) return img;
    }
    if (imf->numsizes >= 10 /* MAXIMSIZES */) return NULL;
    img = &(imf->images[imf->numsizes]);
    ++(imf->numsizes);
    img->w = w;  img->h = h;
    /* Default min/max limits to actual size. */
    img->minw = img->maxw = w;  img->minh = img->maxh = h;
    return img;
}

/* This tries to fill in the given image family. */

load_image_family(imf)
ImageFamily *imf;
{
    int w, h, i;
    Pixmap pic;
    Image *img;

    /* If no name, don't even try. */
    if (imf->name == NULL) return;
    if (0 /* found an imf file */) {
    } else {
#if 0
	/* Just grab up plausibly-named bitmaps. */
	if ((pic = load_bitmap(side, imf->name, "b", &w, &h)) != None) {
	    if ((img = get_img(imf, w, h)) != NULL) {
		img->mono = pic;
	    }
	}
	if ((pic = load_bitmap(side, imf->name, "32.b", &w, &h)) != None) {
	    if ((img = get_img(imf, w, h)) != NULL) {
		img->mono = pic;
	    }
	}
	if ((pic = load_bitmap(side, imf->name, "16.t", &w, &h)) != None) {
	    if ((img = get_img(imf, w, h)) != NULL) {
		img->mono = pic;
	    }
	}
	if ((pic = load_bitmap(side, imf->name, "t", &w, &h)) != None) {
	    if ((img = get_img(imf, w, h)) != NULL) {
		img->mono = pic;
		img->mask = pic;
	    }
	}
	if ((pic = load_bitmap(side, imf->name, "e", &w, &h)) != None) {
	    if ((img = get_img(imf, w, h)) != NULL) {
		img->mono = pic;
		img->mask = pic;
	    }
	}
#endif
#if 0
	if (imf->numsizes > 0) {
	    return TRUE;
	} else {
	    fprintf(stderr,
		    "Image named \"%s\" not found!\n",
		    imf->name, bitmaplib);
	}
#endif
    }
}

/* pbmtoxbm.c - read a portable bitmap and produce an X11 bitmap file
**
** Copyright (C) 1988 by Jef Poskanzer.
**
** Permission to use, copy, modify, and distribute this software and its
** documentation for any purpose and without fee is hereby granted, provided
** that the above copyright notice appear in all copies and that both that
** copyright notice and this permission notice appear in supporting
** documentation.  This software is provided "as is" without express or
** implied warranty.
*/

/* #include "pbm.h" */

write_bitmap(name, ext, cols, rows, datalist)
char *name, *ext;
int cols, rows;
Obj *datalist;
{
    FILE *fp;
    char fname[1000];
/*    bit *bitrow;
    register bit *bP;  */
    int format, padright, row, i;
    register int col;
    char *cp, *datastr;
    int itemsperline;
    register int bitsperitem;
    register int item;
    int firstitem;
    static char hexchar[] = "0123456789abcdef";

    sprintf(fname, "%s/%s.%s", outdirname, name, ext);
    if ((fp = fopen(fname, "w")) != NULL) {
	/* Compute padding to round cols up to the nearest multiple of 8. */
	padright = ( ( cols + 7 ) / 8 ) * 8 - cols;

	fprintf(fp, "#define %s_width %d\n", name, cols );
	fprintf(fp, "#define %s_height %d\n", name, rows );
	fprintf(fp, "static char %s_bits[] = {", name );

	itemsperline = 12;
	bitsperitem = 0;
	item = 0;
	firstitem = 1;

	datastr = NULL;
	i = 1000;
	for (row = 0; row < rows; row++) {
	    for (col = cols; col >= 0; col--) {
		if (datastr == NULL || i >= strlen(datastr)) {
		    if (datalist == lispnil) goto blowoff;
		    datastr = c_string(car(datalist));
		    i = 0;
		    datalist = cdr(datalist);
		}
		if (col % 8 == 0) {
		    if (firstitem) {
			firstitem = 0;
		    } else {
			putc(',', fp);
			putc(' ', fp);
		    }
		    if (itemsperline >= 12) {
			fprintf(fp, "\n   ");
			itemsperline = 0;
		    }
		    fprintf(fp, "0x%c%c", datastr[i], datastr[i+1]);
		    i += 2;
		    ++itemsperline;
		}
	    }
	}
      blowoff:
	fprintf(fp, "};\n" );
	fclose(fp);
    }
}

/* Need these so we can link with main Xconq code. */

announce_read_progress() {}

/* An init error needs to have the command re-run. */

/* (these should mention file and line) */

init_error(str, a1, a2, a3, a4, a5, a6)
char *str;
long a1, a2, a3, a4, a5, a6;
{
    fprintf(stderr, "Error: ");
    fprintf(stderr, str, a1, a2, a3, a4, a5, a6);
    fprintf(stderr, ".\n");
    fflush(stderr);
}

/* Might be sharable among cmdline-based programs. */

abort_init()
{
    fprintf(stderr, "Fatal errors encountered during game setup, exiting\n");
    exit(1);
}

/* A warning just gets displayed, no other action is taken. */

init_warning(str, a1, a2, a3, a4, a5, a6)
char *str;
long a1, a2, a3, a4, a5, a6;
{
    fprintf(stderr, "Warning: ");
    fprintf(stderr, str, a1, a2, a3, a4, a5, a6);
    fprintf(stderr, ".\n");
    fflush(stderr);
}

/* A run error is fatal. */

run_error(str, a1, a2, a3, a4, a5, a6)
char *str;
long a1, a2, a3, a4, a5, a6;
{
    printf("\nError: ");
    printf(str, a1, a2, a3, a4, a5, a6);
    printf("!\n");
    exit(1);
}

/* Runtime warnings are for when it's important to bug the players,
   usually a problem with xconq or a period. */

run_warning(str, a1, a2, a3, a4, a5, a6)
char *str;
long a1, a2, a3, a4, a5, a6;
{
    printf("\nWarning: ");
    printf(str, a1, a2, a3, a4, a5, a6);
    printf("!\n");
    fflush(stdout);
}
