/* $XConsortium: MapP.h,v 1.6 91/03/13 20:12:07 rws Exp $ */

/* Copyright	Massachusetts Institute of Technology	1987, 1988
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of M.I.T. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  M.I.T. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 *
 * M.I.T. DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL M.I.T.
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 */

#ifndef _MapP_h
#define _MapP_h

#include "xtmap.h"

#include <X11/CoreP.h>

/* No map class stuff. */

typedef struct {
    int empty;
} MapClassPart;

typedef struct _MapClassRec {
    CoreClassPart	core_class;
    MapClassPart	map_class;
} MapClassRec;

extern MapClassRec mapClassRec;

/* Lots of private info. */

typedef struct {
    /* resources */
    char* resource;
    /* private state */
    GC gc;                    /* a "scratch" gc - used a lot */
    Map *map;
    Side *side;
    int isworld;
    short curx, cury;         /* current spot being looked at */
    struct a_unit *curunit;   /* unit under cursor */
    short vx, vy;             /* lower left of viewport in hex coords */
    short vw, vh;             /* Boundaries of viewport in hex coords */
    short vw2, vh2;           /* 1/2 (rounded down) of above values */
    short power;              /* index to this map's magnification */
    short mag;                /* magnification of each hex (a power of 2) */
    short vcx, vcy;           /* center of the view */
    short lastvcx, lastvcy;
    short hw, hh;             /* pixel dims of a hex cell */
    short hch;                /* height of hex cell between centers */
    short uw, uh;             /* pixel dims of unit subcell */
    /* How to draw the map. */
    int drawterrain;         /* Display solid color terrain on the map? */
    int drawhexedges;        /* Draw outlines around hexes? */
    int drawhexpats;         /* Draw terrain patterns? */
    int drawunits;           /* Display units on the map? */
    int drawnames;           /* Display unit names/numbers on the map? */
    short oldesttoshow;       /* the relative time of the oldest data */
    short agetofade;          /* point at which to gray out the display */
    short newesttoshow;       /* the relative time of the newest data */
    short fadeterrain;        /* fade world features as well as units? */
#ifdef LEVELS
    short lowestlevel;        /* the lowest level of units to display */
    short highestlevel;       /* the highest level of units to display */
#endif /* LEVELS */
#ifdef LINEAR
    int drawborders;
    int drawconns;
#endif /* LINEAR */
    int drawpeople;
    int drawelevations;
    int drawresources[MAXMTYPES];
    short drawtemp;
    short drawweather;
    /* All the colors used - filled in by display init */
    long bdcolor;             /* color for borders */
    long bgcolor;             /* background color */
    long fgcolor;             /* foreground (text) color */
    long owncolor;            /* color for us (usually black) */
    long altcolor;            /* another color for us (usually blue) */
    long enemycolor;          /* color for them (usually red) */
    long neutcolor;           /* color for fencesitters (usually gray) */
    long graycolor;           /* color for graying out (usually gray) */
    long diffcolor;           /* unusual/distinct color (usually maroon) */
    long goodcolor;           /* color for OKness (usually green) */
    long badcolor;            /* color for non-OKness (usually red) */
    long hexcolor[MAXTTYPES]; /* the color of each terrain type */
    long hexfgcolor[MAXTTYPES]; /* the "fg" color of each terrain type */
    long ourcolors[3];        /* national colors */
    long ournumcolors;
    long ouremblem;          /* pattern for national emblem (pixmap?) */
    long emblems[MAXSIDES];
    long colors[MAXSIDES][3];
    int numcolors[MAXSIDES];
    /* need legend fonts for other things too */
    XFontStruct *cursorfont;      /* font for cursors */
    /* Terrain drawing machinery */
    enum whattouse usewhat[NUMPOWERS][MAXTTYPES];
    Pixmap terrpics[NUMPOWERS][MAXTTYPES]; /* terrain bitmaps */
    char terrchars[NUMPOWERS][MAXTTYPES];
    XFontStruct *terrfonts[NUMPOWERS][MAXTTYPES];
    Pixmap hexpics[NUMPOWERS], bhexpics[NUMPOWERS];
    Pixmap hexchars[NUMPOWERS], bhexchars[NUMPOWERS];
    XFontStruct *hexfonts[NUMPOWERS], *bhexfonts[NUMPOWERS];
    XFontStruct *iconfont;
    /* Unit drawing machinery */
    Pixmap unitpics[NUMPOWERS][MAXUTYPES];	/* unit bitmaps */
    Pixmap unitmasks[NUMPOWERS][MAXUTYPES];
    char unitchars[NUMPOWERS][MAXUTYPES];
    XFontStruct *unitfonts[NUMPOWERS][MAXUTYPES];
    XFontStruct *unitfont;	/* font for unit characters */
    int unitw[NUMPOWERS][MAXUTYPES], unith[NUMPOWERS][MAXUTYPES];
    /* Emblem drawing machinery */
    Pixmap emblempics[MAXSIDES];
    Pixmap emblemmasks[MAXSIDES];
    int embw[MAXSIDES], embh[MAXSIDES];
    /* Map legend drawing machinery */
    XFontStruct *ulegendfonts[NUMPOWERS][MAXUTYPES];
    XFontStruct *flegendfonts[NUMPOWERS][10];  /* num feature types? */
    /* Random stuff */
    Pixmap bombpics[4];		/* mushroom clouds */
    Pixmap hitpics[3];
    Pixmap boxcurs, boxmask;

} MapPart;

typedef struct _MapRec {
    CorePart core;
    MapPart map;
} MapRec;

#endif /* _MapP_h */
